
#ifndef _GNU_SOURCE
#define _GNU_SOURCE 1
#endif

#include "drinkers.h"

#include <stdlib.h>
#include <string.h>
#include <unistd.h>


// ---Constants---
// Describes connection to the SS.
#define KP_SS_NAME "X"
#define KP_SS_ADDRESS "194.85.173.9"  // Public PetrSU SIB
//#define KP_SS_ADDRESS "127.0.0.1"     // Local SIB
#define KP_SS_PORT 10010


int main()
{
    sslog_init();

    sslog_node_t *node = sslog_new_node("KP_Timo", KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);

    register_ontology();

    // In the smart space the name of the KP must be unique, if you try to
    // join without leave, then you received an error.
    if (sslog_node_join(node) != SSLOG_ERROR_NO) {
        sslog_shutdown();
        printf("Can't join to SS: %s\n", sslog_error_get_last_text());
        return 0;
    }

    printf("KP joins to SS.\n");

    printf("\n\n --- Timo's agent have joined to SS --- \n\n\n");
    printf("Initialization of structures...\n");

    sslog_individual_t *timo = sslog_new_individual(CLASS_MAN, "timo");

    sslog_insert_property(timo, PROPERTY_FNAME, "Timo");
    sslog_insert_property(timo, PROPERTY_LNAME, "Ericsson");

    sslog_node_insert_individual(node, timo);

    sslog_individual_t *sms = sslog_new_individual(CLASS_SMS, "sms");

    sslog_insert_property(sms, PROPERTY_TO, "87236482");

    sslog_subscription_t *subscription = sslog_new_subscription(node, true);

    if (subscription == NULL) {
        printf("Error: %s", sslog_error_get_last_text());
        sslog_node_leave(node);
        sslog_shutdown();
        return 0;
    }

    printf("Subscribing to SMS...\n");
	
    list_t *properties = list_new();
    list_add_data(properties, PROPERTY_TO);
    list_add_data(properties, PROPERTY_FROM);

    sslog_sbcr_add_individual(subscription, sms, properties);

    if (sslog_sbcr_subscribe(subscription) != 0) {
            printf("Can't subscribe: %s\n", sslog_error_get_last_text());
            sslog_node_leave(node);
            sslog_shutdown();
	    return 0;
    }

    printf("Subscribing complete.\n");

    char liters[2] = {'0', '\0'};

    printf("\n\n --- Press any key to continue --- \n\n");
    getc(stdin);

    sslog_node_insert_property(node, timo, PROPERTY_DRINKS, "beer");
    sslog_node_insert_property(node, timo, PROPERTY_NUMBER_OF_DRINKS, "0");

    while (true) {
        char old_val[2] = {'\0', '\0'};
        strcpy(old_val, liters);

        const char *p_val = sslog_get_property(sms, PROPERTY_FROM);

        if (p_val != NULL && strcmp(p_val, "732468") == 0) {
            sslog_node_remove_property(node, sms, PROPERTY_FROM, NULL);
            printf("\n\n -!-!- Incoming SMS from 732468 -!-!-\n\n");
            sleep(1);
            printf("\n\n --- Timo have stoped drinking --- \n");
            old_val[0] = '\0';
            break;
        }

        ++liters[0];
        printf("Timo drinks\t%s liters\n", liters);

        if (liters[0] == '5') {
            old_val[0] = '\0';
            break;
        }

        if (sslog_node_update_property(node, timo, PROPERTY_NUMBER_OF_DRINKS, old_val, liters) < 0) {
            printf("\n\n ------- Values coudn't be updated: %s\n", sslog_error_get_last_text());
        }

        sleep(3);
    }

    sslog_node_remove_property(node, timo, PROPERTY_DRINKS, NULL);
    sslog_node_remove_property(node, timo, PROPERTY_NUMBER_OF_DRINKS, NULL);

    sslog_sbcr_unsubscribe(subscription);

    sslog_node_leave(node);

    printf("\n\nTimo's agent leave SS...\n");

    sslog_shutdown();

    return 0;
}
