#define NO_INDIVIDUALS 123

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "drinkers.h"


// ---Constants---
// Describes connection to the SS.
#define KP_SS_NAME "X"
#define KP_SS_ADDRESS "194.85.173.9"  // Public PetrSU SIB
//#define KP_SS_ADDRESS "127.0.0.1"     // Local SIB
#define KP_SS_PORT 10010


int main()
{
    sslog_init();

    sslog_node_t *node = sslog_new_node("KP_Wife", KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);

    register_ontology();

    // In the smart space the name of the KP must be unique, if you try to
    // join without leave, then you received an error.
    if (sslog_node_join(node) != SSLOG_ERROR_NO) {
        printf("Can't join to SS: %s\n", sslog_error_get_last_text());
        return 0;
    }

    printf("\n\n --- Wife's agent have joined to SS --- \n");
    printf("Initialization of structures...\n");
  
    // Get Timos using patterns, now SIB does't correctly query triples
    // with data values: <*> - name - "Timo"
    // pattern_t *pattern = sslog_new_pattern(CLASS_MAN, NULL);
    // sslog_add_property_to_pattern(pattern, PROPERTY_FNAME, "Timo", PATTERN_COND_NO);
    // sslog_add_property_to_pattern(pattern, PROPERTY_LNAME, "Ericsson", PATTERN_COND_NO);
    // list_t *timos = sslog_ss_get_individual_by_pattern_all(pattern);

    // Instead, get Timos wiout patterns.
    list_t *timos = sslog_node_get_individuals_by_class(node, CLASS_MAN);

    if (list_is_null_or_empty(timos) == true) {
        sslog_node_leave(node);
        list_free_with_nodes(timos, NULL);
        printf("There are no such individuals.\n");
        sslog_shutdown();
        return 0;
    }


    sslog_individual_t *timo = NULL;
    list_head_t *pos = NULL;

    list_for_each(pos, &timos->links)
    {
        list_t *node = list_entry(pos, list_t, links);
        timo = (sslog_individual_t *) node->data;
        break;
    }

    list_free_with_nodes(timos, NULL);

    printf("done\n");

    printf("Subscribing to beer...\t\t\t\t");
    sslog_subscription_t *subscription = sslog_new_subscription(node, false);

    list_t *properties = list_new();
    list_add_data(properties, PROPERTY_DRINKS);

    sslog_sbcr_add_individual(subscription, timo, properties);

    if (sslog_sbcr_subscribe(subscription) != 0) {
        printf("\nCan't subscribe\n");
        sslog_free_subscription(subscription);
        sslog_node_leave(node);
        return 0;
    }
    printf("done\n");

    printf("\n\n --- Press any key to continue --- \n\n");
    getc(stdin);

    int sms_sended = 0;
    while (true) {
        const char *p_val = sslog_get_property(timo, PROPERTY_DRINKS);

        if (!sms_sended && p_val != NULL && strcmp(p_val, "beer") == 0) {
            printf("\n\n --- Timo starts drinking --- \n\n");
            printf("Subscribing to number of liters...\n");
            sslog_subscription_t *subscription2 = sslog_new_subscription(node, false);

            list_t *properties2 = list_new();
            list_add_data(properties2, PROPERTY_NUMBER_OF_DRINKS);

            sslog_sbcr_add_individual(subscription2, timo, properties2);

            if (sslog_sbcr_subscribe(subscription2) != 0) {
                printf("\nCan't subscribe\n");
                sslog_free_subscription(subscription2);
                sslog_node_leave(node);
                return 0;
            }
            printf("done\n");

            while (true) {
                const char *p_val2 = sslog_get_property(timo, PROPERTY_NUMBER_OF_DRINKS);
                printf("Timo drinks\t%s liters\n", (char *) p_val2);

                if (p_val2 != NULL && strcmp(p_val2, "3") == 0) {
                    printf("\nSending SMS ... \t\t\t\t");
                    sslog_individual_t *sms = sslog_new_individual(CLASS_SMS, "sms");

                    sslog_insert_property(sms, PROPERTY_TO, "87236482");
                    sslog_insert_property(sms, PROPERTY_FROM, "732468");

                    sslog_node_insert_individual(node, sms);
                    printf("done\n");

                    sms_sended = 1;

                    break;
                }

                sslog_sbcr_wait(subscription2);
            }

        } else if (sms_sended && p_val == NULL) {
            printf("\n\nWait Timo at home :)\n");
            break;
        } else {
            printf("\n\nWait, Timo haven't starts drunk yet\n");
        }
        if (sms_sended) break;

        sslog_sbcr_wait(subscription);
    }

    sleep(3);

    sslog_sbcr_unsubscribe_all(node, false);

    sslog_node_leave(node);

    printf("\nKP leave SS...\n");

    sslog_shutdown();

    return 0;
}
