
#include "hello.h"

// ---Constants---
// Describes connection to the SS.
#define KP_SS_NAME "X"
#define KP_SS_ADDRESS "194.85.173.9"  // Public PetrSU SIB
//#define KP_SS_ADDRESS "127.0.0.1"     // Local SIB
#define KP_SS_PORT 10010


int main()
{
    sslog_init();

    sslog_node_t *node = sslog_new_node("HelloWorldHigh_KP", KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);

    register_ontology(node);

    // In the smart space the name of the KP must be unique, if you try to
    // join without leave, then you received an error.
    if (sslog_node_join(node) != SSLOG_ERROR_NO) {
        printf("Can't join to SS: %s\n", sslog_error_get_last_text());
        return 0;
    }

    printf("KP join to SS\n");

    sslog_individual_t *universum = sslog_new_individual(CLASS_UNIVERSUM, "http://testclass.ru#universum1");

	if (universum == NULL) {
        printf("\nError: %s\n", sslog_error_get_last_text());
		return 0;
	}

    sslog_subscription_t *subcription = sslog_new_subscription(node, false);

    list_t *properties = list_new();
    list_add_data(properties, PROPERTY_HAS);

    sslog_sbcr_add_individual(subcription, universum, properties);

    if (sslog_sbcr_subscribe(subcription) != 0) {
		printf("\nCan't subscribe\n");
		sslog_free_subscription(subcription);
        sslog_node_leave(node);
        return 0;
	}

    const char *p_val = (const char *) sslog_get_property(universum, PROPERTY_HAS);

    if (p_val != NULL) {
        printf("\nNow string is: %s\n", p_val);
	}

	sslog_sbcr_wait(subcription);

    p_val =  (const char *) sslog_get_property(universum, PROPERTY_HAS);

    if (p_val != NULL) {
        printf("\nPublished string: %s\n", p_val);
	}

    sslog_sbcr_unsubscribe_all(node, true);

    sslog_node_leave(node);

    printf("\nKP leave SS...\n");

    sslog_shutdown();
    return 0;
}
