/**
 * @file phone.c
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   Created on July 19, 2015, 12:35 AM
 * @brief  Calling phone KP in media example.
 *
 *
 * @section LICENSE
 *
 * SmartSlog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SmartSlog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartSlog KP Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 *
 * @section DESCRIPTION
 *
 * <b>Simulate activity of the phone.</b> <br />
 * Phone is a part of the Media example.
 * One KP publishes inforamtion about a calling (phone).
 * Another (player) gets information about calling and change sound (On/Off).
 * Player uses subscription to check calls.
 *
 * This file is part of the SmartSlog project.
 *
 * Copyright (C) 2015 by PetrSU SmartSlog team.
 */

// Enable strdup
#ifndef _GNU_SOURCE
#define _GNU_SOURCE 1
#endif

#include "media.h"

#include <stdlib.h>
#include <string.h>


/******************************* Definitions **********************************/
/***************************** Static functions *******************************/
static void print_menu();


/**************************** Implementations *********************************/
/***************************** Main function **********************************/
int main()
{
    sslog_init();

    sslog_node_t *node = sslog_new_node("KP_Phone", KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);

    register_ontology();

    // In the smart space the name of the KP must be unique, if you try to
    // join without leave, then you received an error.
    if (sslog_node_join(node) != SSLOG_ERROR_NO) {
        sslog_shutdown();
        printf("Can't join to SS: %s\n", sslog_error_get_last_text());
        return 0;
    }


    sslog_individual_t *phone = sslog_new_individual(CLASS_MOBILEPHONE, "phone1");

    if (phone == NULL) {
        printf("\nError: %s\n", sslog_error_get_last_text());
		return 0;
	}

	char *calling = (char *) malloc(sizeof(char) * 2);
	calling[0] = '0';
	calling[1] = '\0';

    sslog_insert_property(phone, PROPERTY_ISCALLING, calling);

	while (true) {
		print_menu();

		char c = getc(stdin);

		if (c == '0') {
			break;
		} else if (c == '1') {
            char *old_val = strdup(calling);
			calling[0] = '1';

            if (sslog_node_update_property(node, phone, PROPERTY_ISCALLING, old_val, calling) != SSLOG_ERROR_NO) {
				printf("Values coudn't updated\n");
            }

			free(old_val);
		} else if (c == '2') {
			char *old_val = strdup(calling);
			calling[0] = '0';

            if (sslog_node_update_property(node, phone, PROPERTY_ISCALLING, old_val, calling) != SSLOG_ERROR_NO) {
				printf("Values coudn't updated\n");
            }

			free(old_val);
		}
	}

	free(calling);

    // Clean smart space: remove individual.
    sslog_node_remove_individual(node, phone);

    sslog_node_leave(node);

    sslog_shutdown();

	return 0;
}



/***************************** Implementations ********************************/
/***************************** Static functions *******************************/
static void print_menu()
{
    printf("\n--- Menu ---");
    printf("\n1 - Call incoming.");
    printf("\n2 - Call ends.");
    printf("\n0 - Exit.\n");
}


