/**
 * @file phone.c
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   Created on July 19, 2015, 12:35 AM
 * @brief  Calling phone KP in media example.
 *
 *
 * @section LICENSE
 *
 * SmartSlog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SmartSlog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartSlog KP Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 *
 * @section DESCRIPTION
 *
 * <b>Simulate activity of the player.</b> <br />
 * Player is a part of the Media example.
 * One KP publishes inforamtion about a calling (phone).
 * Another (player) gets information about calling and change sound (On/Off).
 * Player uses subscription to check calls.
 *
 * This file is part of the SmartSlog project.
 *
 * Copyright (C) 2015 by PetrSU SmartSlog team.
 */

// Enable strdup
#ifndef _GNU_SOURCE
#define _GNU_SOURCE 1
#endif

#include "media.h"

#include <stdlib.h>
#include <string.h>


/**************************** Implementations *********************************/
/***************************** Main function **********************************/
int main(int argc, char **argv)
{
    sslog_init();

    sslog_node_t *node = sslog_new_node("KP_Player", KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);

    register_ontology();

    // In the smart space the name of the KP must be unique, if you try to
    // join without leave, then you received an error.
    if (sslog_node_join(node) != SSLOG_ERROR_NO) {
        sslog_shutdown();
        printf("Can't join to SS: %s\n", sslog_error_get_last_text());
        return 0;
    }

    char *uri = (argc < 2) ? "phone1" : argv[1];

    sslog_individual_t *phone = sslog_new_individual(CLASS_MOBILEPHONE, uri);

	if (phone == NULL) {
        printf("\nError: %s\n", sslog_error_get_last_text());
		return 0;
	}

    sslog_subscription_t  *subscription = sslog_new_subscription(node, false);

    list_t *properties = list_new();
    list_add_data(properties, PROPERTY_ISCALLING);

    sslog_sbcr_add_individual(subscription, phone, properties);

    if (sslog_sbcr_subscribe(subscription) != SSLOG_ERROR_NO) {
		printf("\nCan't subscribe\n");
        sslog_node_leave(node);
        sslog_shutdown();
		return 0;
	}

	char *old_val = NULL;
	while (true) {
        char *p_val = (char *) sslog_get_property(phone, PROPERTY_ISCALLING);

        if (p_val == NULL) {
            printf ("No data about calling...\n");
            break;
        }


        char *new_val = strdup((char *) p_val);

        if (old_val == NULL) {
            old_val = strdup(new_val);
        }

        if ((new_val[0]=='0') && (old_val[0]=='1'))
            printf("\n -------- Volume UP\n");
        else if ((new_val[0]=='1') && (old_val[0]=='0'))
            printf("\n -------- Volume DOWN\n");

        free(old_val);
        old_val = new_val;

        printf("\n");

        fflush(stdout);

          sslog_sbcr_wait(subscription);
	}

    free(old_val);

    sslog_sbcr_unsubscribe(subscription);

    sslog_node_leave(node);

    sslog_shutdown();

	return 0;
}
