/*
** Copyright (c) 1990- 1993, 1996 Open Software Foundation, Inc.
** Copyright (c) 1989 by Hewlett-Packard Company, Palo Alto, Ca. &
** Digital Equipment Corporation, Maynard, Mass.
** Copyright (c) 1998 Microsoft.
** To anyone who acknowledges that this file is provided "AS IS"
** without any express or implied warranty: permission to use, copy,
** modify, and distribute this file for any purpose is hereby
** granted without fee, provided that the above copyright notices and
** this notice appears in all source code copies, and that none of
** the names of Open Software Foundation, Inc., Hewlett-Packard
** Company, Microsoft, or Digital Equipment Corporation be used in
** advertising or publicity pertaining to distribution of the software
** without specific, written prior permission. Neither Open Software
** Foundation, Inc., Hewlett-Packard Company, Microsoft, nor Digital
** Equipment Corporation makes any representations about the
** suitability of this software for any purpose.
*/


/*
** In order to obtain the declaration of gethostname from <unistd.h>, 
** the following macros definitions must
** be made before including any header files
*/
#define _BSD_SOURCE
#define _XOPEN_SOURCE
#define PATH_MAX 256

#ifdef SSLOG_WIN
#define SLASH '\\'
#define ISSLASH(C) ((C) == '\\')
#include <io.h>
#else
#define SLASH '/'
# define ISSLASH(C) ((C) == '/')
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "sysdep.h"

/* system dependent call to get IEEE node ID.
   This sample implementation generates a random node ID. */
void get_ieee_node_identifier(uuid_node_t *node)
{
    static int inited = 0;
    static uuid_node_t saved_node;
    char seed[16];
    FILE *fp;
    
    /* Path to temporary file */  
    char* tmpfile = generate_path_temporary_file("uuid_nodeid");
    
    if (tmpfile == NULL)
        return;
    
    if (!inited) {
        fp = fopen(tmpfile, "rb");
        if (fp) {
            fread(&saved_node, sizeof saved_node, 1, fp);
            fclose(fp);
        }
        else {
            get_random_info(seed);
            seed[0] |= 0x01;
            memcpy(&saved_node, seed, sizeof saved_node);
            fp = fopen(tmpfile, "wb");
            if (fp) {
                fwrite(&saved_node, sizeof saved_node, 1, fp);
                fclose(fp);
            }
        }
        
        inited = 1;
    }

    *node = saved_node;
}

    
/* Generate path to temporary file */  
char *generate_path_temporary_file(char *basename)
{        
	char *dir = NULL;
	char *slashdir = NULL;
	char *tmpfile = NULL;

#if defined SSLOG_WIN
	char dirbuf[PATH_MAX];
	DWORD retval;
	retval = GetTempPath(PATH_MAX, dirbuf);
	if (retval > 0 && retval < PATH_MAX) {
		dir = (char *)malloc(strlen(dirbuf) + 1);
		strcpy(dir, dirbuf);
	}
#else
	dir = (char *)malloc(strlen(P_tmpdir) + 1);
	strcpy(dir, P_tmpdir);
#endif

    if (dir == NULL)
        return NULL;
        
	/* 	add trailing slash */
	int dlen = strlen(dir);
	if (!ISSLASH(dir[dlen - 1])) {
	    slashdir = (char *)malloc(dlen + 1 + 1); /* one for extra char, one for trailing zero */
        strcpy(slashdir, dir);
        slashdir[dlen] = SLASH;
        slashdir[dlen + 1] = '\0';
    } else {
        slashdir = (char *)malloc(dlen + 1);
        strcpy(slashdir, dir);
    }
		    
	tmpfile = (char *)calloc(sizeof(char) * (strlen(slashdir) +
                            strlen(basename) + 1),1);

    memset(tmpfile, 0, strlen(tmpfile));
	strncpy(tmpfile, slashdir, strlen(slashdir));
    strncat(tmpfile, basename, strlen(basename));

    free(dir);
    free(slashdir);
    
    return tmpfile;
}

/* system dependent call to get the current system time. Returned as
   100ns ticks since UUID epoch, but resolution may be less than
   100ns. */
#ifdef _WINDOWS_

void get_system_time(uuid_time_t *uuid_time)
{
    ULARGE_INTEGER time;

    /* NT keeps time in FILETIME format which is 100ns ticks since
       Jan 1, 1601. UUIDs use time in 100ns ticks since Oct 15, 1582.
       The difference is 17 Days in Oct + 30 (Nov) + 31 (Dec)
       + 18 years and 5 leap days. */
    GetSystemTimeAsFileTime((FILETIME *)&time);
    time.QuadPart +=

          (unsigned __int64) (1000*1000*10)       // seconds
        * (unsigned __int64) (60 * 60 * 24)       // days
        * (unsigned __int64) (17+30+31+365*18+5); // # of days
    *uuid_time = time.QuadPart;
}

/* Sample code, not for use in production; see RFC 1750 */
void get_random_info(char seed[16])
{
    MD5_CTX c;
    struct {
        MEMORYSTATUS m;
        SYSTEM_INFO s;
        FILETIME t;
        LARGE_INTEGER pc;
        DWORD tc;
        DWORD l;
        char hostname[MAX_COMPUTERNAME_LENGTH + 1];
    } r;

    MD5Init(&c);
    GlobalMemoryStatus(&r.m);
    GetSystemInfo(&r.s);
    GetSystemTimeAsFileTime(&r.t);
    QueryPerformanceCounter(&r.pc);
    r.tc = GetTickCount();
    r.l = MAX_COMPUTERNAME_LENGTH + 1;
    GetComputerName(r.hostname, &r.l);
	MD5Update(&c, (unsigned char *) &r, sizeof r);
	MD5Final((unsigned char *) seed, &c);
}

#else

void get_system_time(uuid_time_t *uuid_time)
{
    struct timeval tp;

    gettimeofday(&tp, (struct timezone *)0);

    /* Offset between UUID formatted times and Unix formatted times.
       UUID UTC base time is October 15, 1582.
       Unix base time is January 1, 1970.*/
    *uuid_time = ((unsigned64)tp.tv_sec * 10000000)
        + ((unsigned64)tp.tv_usec * 10)
        + I64(0x01B21DD213814000);
}

/* Sample code, not for use in production; see RFC 1750 */
void get_random_info(char seed[16])
{
    MD5_CTX c;
    struct {
        struct sysinfo s;
        struct timeval t;
        char hostname[257];
    } r;

    MD5Init(&c);
    sysinfo(&r.s);
    gettimeofday(&r.t, (struct timezone *)0);
    gethostname(r.hostname, 256);
    MD5Update(&c, (unsigned char *) &r, sizeof r);
    MD5Final((unsigned char *) seed, &c);
}

#endif

