
#include <stdlib.h>
#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>

#include "Basic.h"

#include "../src/utils/errors_internal.h"
#include "../src/utils/errors_decl.h"


sslog_error_t *error1;
sslog_error_t *error2;


/* The suite initialization function.
 * Opens the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int init_suite1(void)
{
    error1 = (sslog_error_t *) malloc(sizeof(sslog_error_t));

    if (error1 == NULL) {
        return -1;
    }

    error2 = (sslog_error_t *) malloc(sizeof(sslog_error_t));

    if (error2 == NULL) {
        return -1;
    }

    return 0;
}

/* The suite cleanup function.
 * Closes the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int clean_suite1(void)
{
    free(error1);
    free(error2);

    return 0;
}


void test_last_error_set_null_comment()
{
    sslog_error_set(NULL, -1, NULL);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), -1);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), "\0");
}



void test_last_error_set()
{
    sslog_error_set(NULL, SSLOG_ERROR_INCORRECT_ARGUMENT,
                    SSLOG_ERROR_TEXT_INCORRECT_ARGUMENT "test argument is null.");

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),
                           SSLOG_ERROR_TEXT_INCORRECT_ARGUMENT "test argument is null.");

    sslog_error_set(NULL, SSLOG_ERROR_OUT_OF_MEMORY,
                    SSLOG_ERROR_TEXT_OUT_OF_MEMORY);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_OUT_OF_MEMORY);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),
                           SSLOG_ERROR_TEXT_OUT_OF_MEMORY);

}

void test_last_error_reset()
{
    sslog_error_set(NULL, SSLOG_ERROR_NOT_IMPLEMENTED, SSLOG_ERROR_TEXT_NOT_IMPLEMENTED);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NOT_IMPLEMENTED);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),
                           SSLOG_ERROR_TEXT_NOT_IMPLEMENTED);

    sslog_error_reset(NULL);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),SSLOG_ERROR_TEXT_NO);
}


void test_error_struct_set()
{
    sslog_error_set(error1, SSLOG_ERROR_INCORRECT_CLASS,
                    SSLOG_ERROR_TEXT_INCORRECT_CLASS "Myclass");

    CU_ASSERT_EQUAL(error1->code, SSLOG_ERROR_INCORRECT_CLASS);
    CU_ASSERT_STRING_EQUAL(error1->text,
                           SSLOG_ERROR_TEXT_INCORRECT_CLASS "Myclass");

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), error1->code);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), error1->text);


    sslog_error_set(error2, SSLOG_ERROR_INCORRECT_CLASSTYPE,
                    SSLOG_ERROR_TEXT_INCORRECT_CLASSTYPE);

    CU_ASSERT_EQUAL(error2->code, SSLOG_ERROR_INCORRECT_CLASSTYPE);
    CU_ASSERT_STRING_EQUAL(error2->text, SSLOG_ERROR_TEXT_INCORRECT_CLASSTYPE);

    CU_ASSERT_NOT_EQUAL(error1->code, error2->code);
    CU_ASSERT_STRING_NOT_EQUAL(error1->text, error2->text);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), error2->code);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), error2->text);


    sslog_error_set(error1, SSLOG_ERROR_INCORRECT_CLASSTYPE,
                    SSLOG_ERROR_TEXT_INCORRECT_CLASSTYPE);

    CU_ASSERT_EQUAL(error1->code, error2->code);
    CU_ASSERT_STRING_EQUAL(error1->text, error2->text);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), error1->code);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), error1->text);

}

void test_error_struct_reset()
{
    sslog_error_set(error1, SSLOG_ERROR_NOT_IMPLEMENTED, SSLOG_ERROR_TEXT_NOT_IMPLEMENTED);
    sslog_error_set(error2, SSLOG_ERROR_CANT_SUBSCRIBE, SSLOG_ERROR_TEXT_CANT_SUBSCRIBE);

    sslog_error_reset(error1);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),SSLOG_ERROR_TEXT_NO);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), error1->code);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), error1->text);

    CU_ASSERT_EQUAL(error2->code, SSLOG_ERROR_CANT_SUBSCRIBE);
    CU_ASSERT_STRING_EQUAL(error2->text, SSLOG_ERROR_TEXT_CANT_SUBSCRIBE);

    sslog_error_set(error1, SSLOG_ERROR_NOT_IMPLEMENTED, SSLOG_ERROR_TEXT_NOT_IMPLEMENTED);
    sslog_error_reset(error2);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(),SSLOG_ERROR_TEXT_NO);

    CU_ASSERT_EQUAL(sslog_error_get_last_code(), error2->code);
    CU_ASSERT_STRING_EQUAL(sslog_error_get_last_text(), error2->text);
}








void testTriple()
{
}


/* The main() function for setting up and running the tests.
 * Returns a CUE_SUCCESS on successful running, another
 * CUnit error code on failure.
 */
int main()
{
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("Suite_1", init_suite1, clean_suite1);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    /* NOTE - ORDER IS IMPORTANT - MUST TEST fread() AFTER fprintf() */
    if ((NULL == CU_add_test(pSuite, "test of NULL triple", test_last_error_set)) ||
            (NULL == CU_add_test(pSuite, "Test reset error", test_last_error_reset)) ||
            (NULL == CU_add_test(pSuite, "Error struct setting", test_error_struct_set)) ||
            (NULL == CU_add_test(pSuite, "Null comment setting", test_last_error_set_null_comment)) ||
            (NULL == CU_add_test(pSuite, "Error struct reset", test_error_struct_reset)))
    {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();

    int failed_tests = CU_get_number_of_tests_failed();

    CU_cleanup_registry();

    return failed_tests;
}
