
#include <stdlib.h>
#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>

#include "include/Basic.h"

#include "session_internal.h"

#include "utils/errors_internal.h"
#include "utils/errors_decl.h"


sslog_error_t *error1;
sslog_error_t *error2;

/* The suite initialization function.
 * Opens the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int init_suite_nodes(void)
{
    return 0;// sslog_init();
}

/* The suite cleanup function.
 * Closes the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int clean_suite_nodes(void)
{
    sslog_shutdown();

    return 0;
}


/* Creates new nodes with different NULL or incorrect parameters and checks errors.
 * Creates and deletes nodes with the same ID in one session and checks error.
 * (in one seesion must be only node with uniq ID).
 * Creates nodes in different session.
 */
void test_node_new_free()
{
    sslog_node_t *node = sslog_new_node(NULL, NULL, NULL, -1);
    CU_ASSERT_PTR_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    node = sslog_new_node("nodeId", NULL, NULL, -1);
    CU_ASSERT_PTR_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    node = sslog_new_node("nodeId", "X", NULL, -1);
    CU_ASSERT_PTR_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    node = sslog_new_node("nodeId", "X", "10.10.10.10", -1);
    CU_ASSERT_PTR_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    node = sslog_new_node("nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_node_t *node_same_id = sslog_new_node("nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NULL(node_same_id);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    sslog_free_node(node);

    node_same_id = sslog_new_node("nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node_same_id);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_free_node(node_same_id);
}



void test_sessions_and_nodes()
{
    sslog_session_t *session1 = sslog_new_session();
    CU_ASSERT_PTR_NOT_NULL(session1);

    sslog_session_t *session2 = sslog_new_session();
    CU_ASSERT_PTR_NOT_NULL(session2);

    sslog_node_t *node = sslog_new_node("nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_node_t *node_session1 = sslog_new_session_node(session1, "nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node_session1);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_node_t *node_session2 = sslog_new_session_node(session2, "nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node_session2);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_node_t *node_session3 = sslog_new_session_node(session2, "nodeId3", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node_session3);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);


    sslog_free_session(session2);

    node_session2 = sslog_new_session_node(session1, "nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NULL(node_session2);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_INCORRECT_ARGUMENT);

    node_session2 = sslog_new_session_node(NULL, "nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NULL(node_session2);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NULL_ARGUMENT);

    sslog_free_node(node_session1);

    node_session2 = sslog_new_session_node(session1, "nodeId", "X", "10.10.10.10", 10010);
    CU_ASSERT_PTR_NOT_NULL(node_session2);
    CU_ASSERT_EQUAL(sslog_error_get_last_code(), SSLOG_ERROR_NO);

    sslog_free_node(node);
    sslog_free_session(session1);
}


/* The main() function for setting up and running the tests.
 * Returns a CUE_SUCCESS on successful running, another
 * CUnit error code on failure.
 */
int main()
{
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("Suite: session and nodes", init_suite_nodes, clean_suite_nodes);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }



    /* add the tests to the suite */
    /* NOTE - ORDER IS IMPORTANT - MUST TEST fread() AFTER fprintf() */
    if ((NULL == CU_add_test(pSuite, "new/free nodes", test_node_new_free)) ||
        (NULL == CU_add_test(pSuite, "new/free nodes in sessions", test_sessions_and_nodes)))
    {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();

    int failed_tests = CU_get_number_of_tests_failed();

    CU_cleanup_registry();

    return failed_tests;
}
