
#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>

#include "Basic.h"

#include "triple.h"
#include "kpi_api.h"
#include "kpi_interface.h"
#include "triplestore.h"


sslog_store_t *store;


/* The suite initialization function.
 * Opens the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int init_suite1(void)
{
   store = sslog_store_new();

   return (NULL == store) ? -1 : 0;
}

/* The suite cleanup function.
 * Closes the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int clean_suite1(void)
{
   sslog_store_free(store);

   return 0;
}


void test_store_adding()
{
    sslog_triple_t *triple = sslog_kpi_new_triple(
                                "http://test#subject", "http://test#predicate", "http://test#object",
                                SSLOG_RDF_TYPE_URI);

    CU_ASSERT_PTR_NOT_NULL_FATAL(triple);

    CU_ASSERT_EQUAL(0, sslog_store_add_triple(store, triple));
}


void testTripleNull()
{
    //sslog_triple_t *triple = NULL;

//    CU_ASSERT_PTR_NULL(sslog_triple_get_subject(triple));
//    CU_ASSERT_PTR_NULL(sslog_triple_get_predicate(triple));
//    CU_ASSERT_PTR_NULL(sslog_triple_get_object(triple));
//    CU_ASSERT_EQUAL(sslog_triple_get_subject_type(triple), SSLOG_RDF_TYPE_INCORRECT);
//    CU_ASSERT_EQUAL(sslog_triple_get_object_type(triple), SSLOG_RDF_TYPE_INCORRECT);
}



void testTriple()
{
    char *subject = "http://test.ru#TestTriple-1";
    char *subject_bnode = "http://test.ru#TestBnode-1";
    char *predicate = "http://test.ru#TestPredicate-1";
    char *object_uri = "http://test.ru#TestObject-1";
    char *object_bnode = "http://test.ru#TestBnode-2";
    char *object_lit = "data-1";

    sslog_triple_t *triple_uri = sslog_kpi_new_triple(subject, predicate, object_uri, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_uri);

    CU_ASSERT_STRING_EQUAL(triple_uri->subject, subject);
    CU_ASSERT_STRING_EQUAL(triple_uri->predicate, predicate);
    CU_ASSERT_STRING_EQUAL(triple_uri->object, object_uri);
    CU_ASSERT_EQUAL(triple_uri->subject_type, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_EQUAL(triple_uri->object_type, SSLOG_RDF_TYPE_URI);

    sslog_free_triple(triple_uri);


    sslog_triple_t *triple_lit = sslog_kpi_new_triple(subject, predicate, object_lit, SSLOG_RDF_TYPE_LIT);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_lit);

//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_subject(triple_lit), subject);
//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_predicate(triple_lit), predicate);
//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_object(triple_lit), object_lit);
//    CU_ASSERT_EQUAL(sslog_triple_get_subject_type(triple_lit), SSLOG_RDF_TYPE_URI);
//    CU_ASSERT_EQUAL(sslog_triple_get_object_type(triple_lit), SSLOG_RDF_TYPE_LIT);

    sslog_free_triple(triple_lit);


    sslog_triple_t *triple_bnode = sslog_kpi_new_triple(subject_bnode, predicate, object_bnode, SSLOG_RDF_TYPE_BNODE);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_lit);

//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_subject(triple_bnode), subject_bnode);
//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_predicate(triple_bnode), predicate);
//    CU_ASSERT_STRING_EQUAL(sslog_triple_get_object(triple_bnode), object_bnode);
//    CU_ASSERT_EQUAL(sslog_triple_get_subject_type(triple_bnode), SSLOG_RDF_TYPE_BNODE);
//    CU_ASSERT_EQUAL(sslog_triple_get_object_type(triple_bnode), SSLOG_RDF_TYPE_BNODE);

    sslog_free_triple(triple_bnode);
}


/* The main() function for setting up and running the tests.
 * Returns a CUE_SUCCESS on successful running, another
 * CUnit error code on failure.
 */
int main()
{
   CU_pSuite pSuite = NULL;

   /* initialize the CUnit test registry */
   if (CUE_SUCCESS != CU_initialize_registry())
      return CU_get_error();

   /* add a suite to the registry */
   pSuite = CU_add_suite("Suite_1", init_suite1, clean_suite1);
   if (NULL == pSuite) {
      CU_cleanup_registry();
      return CU_get_error();
   }

   /* add the tests to the suite */
   /* NOTE - ORDER IS IMPORTANT - MUST TEST fread() AFTER fprintf() */
   if ((NULL == CU_add_test(pSuite, "test of NULL triple", testTripleNull)) ||
       (NULL == CU_add_test(pSuite, "test of triplestore()", test_store_adding)))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }

   /* Run all tests using the CUnit Basic interface */
   CU_basic_set_mode(CU_BRM_VERBOSE);
   CU_basic_run_tests();

   int failed_tests = CU_get_number_of_tests_failed();

   CU_cleanup_registry();

   return failed_tests;
}
