
#include "tests.h"

#include "triplestore.h"

#include <stdarg.h>
 #include <stddef.h>
 #include <setjmp.h>

#include "utils/errors.h"

#include "Basic.h"

/* The suite initialization function.
 * Init SmartSlog library.
 * Returns zero on success, non-zero otherwise.
 */
int init_suite_triples(void)
{
    int result = sslog_init();

    if (result != SSLOG_ERROR_NO) {
        return 0;
    }

    return 0;
}

/* The suite cleanup function.
 * Shutdown SmatSlog library/
 * Returns zero on success, non-zero otherwise.
 */
int clean_suite_triples(void)
{
    sslog_shutdown();

    return 0;
}




void test_triple_new()
{
    sslog_triple_t *triple = sslog_new_triple(SSLOG_TEST_URI "subject", SSLOG_TEST_URI "predicate", SSLOG_TEST_URI "object",
                                              SSLOG_RDF_TYPE_URI, SSLOG_RDF_TYPE_URI);

    CU_ASSERT_PTR_NOT_NULL_FATAL(triple);

    CU_ASSERT_STRING_EQUAL(triple->subject, SSLOG_TEST_URI "subject");
    CU_ASSERT_STRING_EQUAL(triple->predicate, SSLOG_TEST_URI "predicate");
    CU_ASSERT_STRING_EQUAL(triple->object, SSLOG_TEST_URI "object");
    CU_ASSERT_EQUAL(triple->subject_type, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_EQUAL(triple->object_type, SSLOG_RDF_TYPE_URI);

    sslog_triple_t *triple_detached = sslog_new_triple_detached(SSLOG_TEST_URI "subject", SSLOG_TEST_URI "predicate", SSLOG_TEST_URI "object",
                                                                SSLOG_RDF_TYPE_URI, SSLOG_RDF_TYPE_URI);

    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_detached);

    CU_ASSERT_STRING_EQUAL(triple_detached->subject, SSLOG_TEST_URI "subject");
    CU_ASSERT_STRING_EQUAL(triple_detached->predicate, SSLOG_TEST_URI "predicate");
    CU_ASSERT_STRING_EQUAL(triple_detached->object, SSLOG_TEST_URI "object");
    CU_ASSERT_EQUAL(triple_detached->subject_type, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_EQUAL(triple_detached->object_type, SSLOG_RDF_TYPE_URI);

    sslog_free_triple(triple_detached);
}


void test_triple_types_new()
{
    char *subject = SSLOG_TEST_URI "TestTriple-1";
    char *subject_bnode =  SSLOG_TEST_URI "TestBnode-1";
    char *predicate = SSLOG_TEST_URI "TestPredicate-1";
    char *object_uri = SSLOG_TEST_URI "TestObject-1";
    char *object_bnode = SSLOG_TEST_URI "TestBnode-2";
    char *object_lit = "data-1";

    sslog_triple_t *triple_uri = sslog_new_triple(subject, predicate, object_uri,
                                                  SSLOG_RDF_TYPE_URI, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_uri);

    CU_ASSERT_STRING_EQUAL(triple_uri->subject, subject);
    CU_ASSERT_STRING_EQUAL(triple_uri->predicate, predicate);
    CU_ASSERT_STRING_EQUAL(triple_uri->object, object_uri);
    CU_ASSERT_EQUAL(triple_uri->subject_type, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_EQUAL(triple_uri->object_type, SSLOG_RDF_TYPE_URI);


    sslog_triple_t *triple_lit = sslog_new_triple(subject, predicate, object_lit,
                                                  SSLOG_RDF_TYPE_URI, SSLOG_RDF_TYPE_LIT);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_lit);

    CU_ASSERT_STRING_EQUAL(triple_lit->subject, subject);
    CU_ASSERT_STRING_EQUAL(triple_lit->predicate, predicate);
    CU_ASSERT_STRING_EQUAL(triple_lit->object, object_lit);
    CU_ASSERT_EQUAL(triple_lit->subject_type, SSLOG_RDF_TYPE_URI);
    CU_ASSERT_EQUAL(triple_lit->object_type, SSLOG_RDF_TYPE_LIT);


    sslog_triple_t *triple_bnode = sslog_new_triple(subject_bnode, predicate, object_bnode,
                                                    SSLOG_RDF_TYPE_BNODE, SSLOG_RDF_TYPE_BNODE);
    CU_ASSERT_PTR_NOT_NULL_FATAL(triple_lit);

    CU_ASSERT_STRING_EQUAL(triple_bnode->subject, subject_bnode);
    CU_ASSERT_STRING_EQUAL(triple_bnode->predicate, predicate);
    CU_ASSERT_STRING_EQUAL(triple_bnode->object, object_bnode);
    CU_ASSERT_EQUAL(triple_bnode->subject_type, SSLOG_RDF_TYPE_BNODE);
    CU_ASSERT_EQUAL(triple_bnode->object_type, SSLOG_RDF_TYPE_BNODE);
}


/* The main() function for setting up and running the tests.
 * Returns a CUE_SUCCESS on successful running, another
 * CUnit error code on failure.
 */
int main()
{
   CU_pSuite pSuite = NULL;

   /* initialize the CUnit test registry */
   if (CUE_SUCCESS != CU_initialize_registry())
      return CU_get_error();

   /* add a suite to the registry */
   pSuite = CU_add_suite("Triple SUIT", init_suite_triples, clean_suite_triples);
   if (NULL == pSuite) {
      CU_cleanup_registry();
      return CU_get_error();
   }

   /* add the tests to the suite */
   /* NOTE - ORDER IS IMPORTANT - MUST TEST fread() AFTER fprintf() */
   if ((NULL == CU_add_test(pSuite, "New triple", test_triple_new)) ||
       (NULL == CU_add_test(pSuite, "New triple of different types", test_triple_types_new)))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }

   /* Run all tests using the CUnit Basic interface */
   CU_basic_set_mode(CU_BRM_VERBOSE);
   CU_basic_run_tests();

   int failed_tests = CU_get_number_of_tests_failed();

   CU_cleanup_registry();

   return failed_tests;
}
