﻿//  ------------------------------------------------------------------------
//  <copyright file="ActionType.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;

namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Action determines the type of change that was made with the object (property, individual, triple etc).
    /// There are two tipes of actions: local (change on local side) and remote (change on smart space side). 
    /// </summary>
    [Flags]
    public enum ActionType
    {
        /// <summary>
        /// Object was removed. 
        /// </summary>
        Remove = RemoteRemove | LocalRemove,

        /// <summary>
        /// Object was inserted.
        /// </summary>
        Insert = RemoteInsert | LocalInsert,

        /// <summary>
        /// Object was updated.
        /// </summary>
        Update = RemoteUpdate | LocalUpdate,

        /// <summary>
        /// 
        /// </summary>
        RemoteRemove = 1 << 0,
        RemoteInsert = 1 << 1,
        RemoteUpdate = 1 << 2,

        LocalRemove = 1 << 3,
        LocalInsert = 1 << 4,
        LocalUpdate = 1 << 5,

        /// <summary>
        /// Unknown action means that no changes were made or it is impossible to determine.
        /// </summary>
        Unknown = 0
    }
}
