﻿//  ------------------------------------------------------------------------
//  <copyright file="DefaultHandler.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------
namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Default handler to build rules for a <see cref="GeneralProperty"/>.
    /// It translates actions under the <see cref="GeneralProperty"/> 
    /// to all properties from the <see cref="IHandlerConfig"/>.
    /// </summary>
    /// <seealso cref="GeneralProperty"/>
    /// <seealso cref="IHandlerConfig"/>
    public class DefaultHandler : Handler
    {
        /// <summary>
        /// Configures rules for insert actions.
        /// </summary>
        /// <param name="config">Config for rules.</param>
        protected override void ConfigureInsert(IHandlerConfig config)
        {
            foreach (Property p in config.Properties)
            {
                config.ToInsert(config.Individual, p, config.NewData);
            }
        }

        /// <summary>
        /// Configures rules for remove actions.
        /// </summary>
        /// <param name="config">Config for rules.</param>
        protected override void ConfigureRemove(IHandlerConfig config)
        {
            foreach (Property p in config.Properties)
            {
                config.ToRemove(config.Individual, p, config.OldData);
            }
        }

        /// <summary>
        /// Configures rules for update actions.
        /// </summary>
        /// <param name="config">Config for rules.</param>
        protected override void ConfigureUpdate(IHandlerConfig config)
        {
            foreach (Property p in config.Properties)
            {
                config.ToUpdate(config.Individual, p, config.NewData, config.OldData);
            }
        }
    }
}
