﻿//  ------------------------------------------------------------------------
//  <copyright file="ExceptionManager.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.IO;

namespace PetrSU.SmartSlog
{
    internal static class ExceptionManager
    {

        internal static string NotJoinMessage
        {
            get
            {
                return "You need to join to the smart space to perform this operation.";
            }
        }

        internal const string EntityNotRegisteredMessage = "Entity is not registered in the session";

        internal const string EntityUriNotRegisteredMessage = "Entity ({0}) is not registered in the session";

        internal const string IncorrectPropertyValueMessage = "Incorrect value of the property. Value can be an individual for object properties and string for data properties.";

        public const string IncorrectPropertyForIndividual = "Property ({0}) cannot be set for the individual.";

        public const string CardinalityMinimumMessage = "Minimum cardinality violation is expected.";

        internal static void Log(int managementType, string methodName, Exception e)
        {
            LoggerManager.WriteExceptionMessage(managementType, methodName, e);
        }

        internal static Exception CreateArgumentNotRegisteredException(string argumentName)
        {
            return new ArgumentException("Entity is not registered in the session", argumentName);
        }

        internal static Exception CreateArgumentNotRegisteredException(string argumentName, OntologyEntity entity)
        {
            return new ArgumentException("Entity is not registered in the session.\nEntity: " + entity, argumentName);
        }

        internal static Exception CreateInvalidOperationNotJoinException()
        {
            return new ArgumentException(NotJoinMessage);
        }


        internal static Exception GetExeption(int smartslogErrorNo) 
        {
            if (smartslogErrorNo >= Constant.SSLOG_NETWORK_ERROR_START && smartslogErrorNo < Constant.SSLOG_NETWORK_ERROR_END)
            {
                return new IOException(NativeMethods.sslog_error_get_last_text());
            }

            return new Exception(NativeMethods.sslog_error_get_last_text());
            //catch (SmartSpaceException ex)
            //{
            //    //throw ex;
            //}
            //catch (KpiException ex)
            //{
            //    //throw new InvalidOperationException("KPI failed: " + ex.ToString());
            //}

        }
    }

}
