﻿//  ------------------------------------------------------------------------
//  <copyright file="HandlerConfig.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System.Collections.Generic;

namespace PetrSU.SmartSlog
{
    class HandlerConfig : IHandlerConfig
    {
        IList<PropertyChangesData> propertiesChanges = new List<PropertyChangesData>();
        IList<PropertyChangesData> localChanges = new List<PropertyChangesData>();
        IList<PropertyChangesData> remoteChanges = new List<PropertyChangesData>();
        IList<Individual> individuals = new List<Individual>();

        internal HandlerConfig(Individual individual, IEnumerable<Property> properties, object newData, object oldData)
        {
            this.Individual = individual;
            this.Properties = new List<Property>(properties);
            this.NewData = newData;
            this.OldData = oldData;
            this.individuals.Add(individual);
        }


        public Individual Individual { get; private set; }

        public IList<Property> Properties { get; private set; }
        public IList<Individual> Individuals { get{return this.individuals;} }
        public object NewData { get; private set; }
        public object OldData { get; private set; }

        internal IList<PropertyChangesData> Changes { get { return propertiesChanges; } }
        //internal IList<PropertyChangesData> LocalChanges { get { return localChanges; } }
        //internal IList<PropertyChangesData> RemoteChanges { get { return remoteChanges; } }

        public void ToInsert(Individual individual, Property property, object data)
        {
            PropertyChangesData changes = new PropertyChangesData(individual, property, data, null, ActionType.Insert);
            //            localChanges.Add(changes);
            propertiesChanges.Add(changes);
            
            if (property.IsObject())
            {
                this.Add(data as Individual);
            }

        }

        public void ToRemove(Individual individual, Property property, object data)
        {
            PropertyChangesData changes = new PropertyChangesData(individual, property, null, data, ActionType.Remove);
            //localChanges.Add(changes);
            propertiesChanges.Add(changes);
     
            if (property.IsObject())
            {
                this.Add(data as Individual);
            }
        }

        public void ToUpdate(Individual individual, Property property, object oldData, object newData)
        {
            PropertyChangesData changes = new PropertyChangesData(individual, property, newData, oldData, ActionType.Update);
            //          localChanges.Add(changes);
            propertiesChanges.Add(changes);
            
            if (property.IsObject())
            {
                this.Add(oldData as Individual);
                this.Add(newData as Individual);
            }
        }

        private void Add(Individual individual)
        {
            if (!individuals.Contains(individual))
            {
                individuals.Add(individual);
            }
        }

    }
}
