﻿//  ------------------------------------------------------------------------
//  <copyright file="ISession.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------
namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Interface of the SmartSlog Session. It manipulates with SSAP sessios, 
    /// each SSAP session is represented by <see cref="Node"/> class.
    /// The session contains: nodes, ontology entities, subscripiton. 
    /// </summary>
    public interface ISession
    {
        /// <summary>
        /// Gets a session identifier.
        /// </summary>
        string Id
        {
            get;
        }

        /// <summary>
        /// Register a <see cref="PetrSU.SmartSlog.OntologyClass"/> in the session. 
        /// </summary>
        /// <param name="ontologyClass">Class to register.</param>
        void RegisterEntity(OntologyClass ontologyClass);

        /// <summary>
        /// Register a <see cref="PetrSU.SmartSlog.Property"/> in the session. 
        /// </summary>
        /// <param name="ontologyProperty">Property to register.</param>
        void RegisterEntity(Property ontologyProperty);

        /// <summary>
        /// Unregister a <see cref="PetrSU.SmartSlog.Node"/>.  
        /// </summary>
        /// <param name="node">Node to unregister.</param>
        void UnregisterNode(Node node);

        /// <summary>
        /// Unregister a <see cref="PetrSU.SmartSlog.OntologyClass"/> from the session, 
        /// after removing you cannot use it with operations. You cannot unregister the calss, 
        /// if it has instances (individuals that are have this parent calss) in the current session or 
        /// if it is used for some subscription.
        /// </summary>
        /// <param name="ontologyClass">ontologyClass to unregister.</param>
        //void UnregisterEntity(OntologyClass ontologyClass);

        /// <summary>
        /// Unregister a <see cref="PetrSU.SmartSlog.Property"/> from the session, 
        /// after removing you cannot use it with operations. You cannot unregister 
        /// the  property if it set for some individuals in the current session or 
        /// if it is used for some subscription.
        /// </summary>
        /// <param name="property">Property to unregister.</param>
        //void UnregisterEntity(Property property);

        /// <summary>
        /// Unregister a <see cref="PetrSU.SmartSlog.Individual"/> from the session, 
        /// after removing you cannot use it with operations. You cannot unregister 
        /// the individual if there are links to this individual 
        /// through properties to thise individual or if it is used for some subscription.
        /// </summary>
        /// <param name="individual">Individual to unregister.</param>
        void UnregisterEntity(Individual individual);

        /// <summary>
        /// Removes triple from local store
        /// </summary>
        /// <param name="triple">Triple to remove</param>
        void RemoveTriple(Triple triple);
        
        /// <summary>
        /// Create triple with setted properties 
        /// </summary>
        /// <param name="subject"></param>
        /// <param name="predicate"></param>
        /// <param name="objectElement"></param>
        /// <param name="subjectType"></param>
        /// <param name="objectType"></param>
        /// <returns></returns>
        Triple CreateTriple(string subject, string predicate, string objectElement, RdfType subjectType, RdfType objectType);

        /// <summary>
        /// Create detached triple with setted properties.
        /// <para>Standalone triple does not stored in the internal triple-store and must be deleted manually.
        /// This function can create triple-template with 'any value' (#SSLOG_TRIPLE_ANY).</para>
        /// </summary>
        /// <param name="subject"></param>
        /// <param name="predicate"></param>
        /// <param name="objectElement"></param>
        /// <param name="subjectType"></param>
        /// <param name="objectType"></param>
        /// <returns></returns>
        Triple CreateDetachedTriple(string subject, string predicate, string objectElement, RdfType subjectType, RdfType objectType);

        /// <summary>
        /// Unregister a <see cref="PetrSU.SmartSlog.Individual"/> from the session, 
        /// after removing you cannot use it with operations. You can unregister 
        /// individual and remove all links from other individuals to this class 
        /// using parameter flag. 
        /// You cannot unregister if it is used for some subscription.
        /// </summary>
        /// <param name="individual">Individual to unregister.</param>
        /// <param name="withLinks">Flag to uregister individual and remove all links (properties) to it.</param>
        //void UnregisterEntity(Individual individual, bool withLinks);
    }
}
