﻿//  ------------------------------------------------------------------------
//  <copyright file="ISubscriptionChangesData.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System.Collections.ObjectModel;

namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Interface that describes changes that was made after receiving an indication 
    /// from the smart space or after successfull subscription (first synchronization).
    /// </summary>
    public interface ISubscriptionChangesData
    {
        /// <summary>
        /// Gets sequence numaber of changes. It is a number of indication that was used as data 
        /// for building changes. Changes that were made after subscription, after firts sinchronization 
        /// has always 0 for this number.  
        /// </summary>
        int SequenceNumber { get; }

        SparqlResult SparqlNewResult { get; }

        SparqlResult SparqlOldResult { get; }

        /// <summary>
        /// Gets individuals by action. Individuals can be inserted (to the smart space), removed or updated.
        /// Updating means that an individual has properties that were changed. 
        /// </summary>
        /// <param name="action">Type of the action: inserting/removing/updating.</param>
        /// <returns>List with individuals or empty list if there are no individuals.</returns>
        ReadOnlyCollection<Individual> GetIndividuals(ActionType action);

        /// <summary>
        /// Gets all individuals that have been inserted, updated or removed in the smart space.
        /// Updating means that an individual has properties that were changed. 
        /// </summary>
        /// <returns>List with individuals or empty list if there are no individuals.</returns>
        ReadOnlyCollection<Individual> GetIndividuals();

        /// <summary>
        /// Gets list with removed individuals uri.
        /// </summary>
        /// <returns>List with individuals uri.</returns>
        ReadOnlyCollection<string> GetRemovedIndividualsUri();


        /// <summary>
        /// Gets list with triples by action. Triples can be inserted, removed or updated. 
        /// </summary>
        /// <param name="action">Action that were perfom under triples.</param>
        /// <returns>List with triples.</returns>
        /// <seealso cref="PetrSU.SmartSlog.Triples" />
        /// <seealso cref="PetrSU.SmartSlog.ActionType" />
        /// <exception cref="System.InvalidOperationException">Thrown when there is no a list for the given action.</exception>
        ReadOnlyCollection<ITriple> GetTriples(ActionType action);

        /// <summary>
        /// Gets list with all triples that were added to the changes. The list contains 
        /// inserted, removed or updated triples. 
        /// </summary>
        /// <returns>List with all triples.</returns>
        ReadOnlyCollection<ITriple> GetTriples();

        ///// <summary>
        ///// Gets new sparql result that were added with changes.
        ///// </summary>
        ///// <returns>New sparql result</returns>
        //SparqlResult GetNewSparqlResult();

        ///// <summary>
        ///// Gets old sparql result that were added with previous changes.
        ///// </summary>
        ///// <returns>Old sparql result</returns>
        //SparqlResult GetOldSparqlResult();

        /// <summary>
        /// Gets changes of properties for an individual by given action. 
        /// Action is a type of changes - property was set/removed/updated.
        /// </summary>
        /// <param name="individual">Individual that has changed properties.</param>
        /// <param name="action">Type of the action taht was made.</param>
        /// <returns>List with properties changes or empty list if there are no properties.</returns>
        ReadOnlyCollection<PropertyChangesData> GetProperties(Individual individual, ActionType action);

        /// <summary>
        /// Gets all changed (set/removed/updated) properties for an individual.
        /// </summary>
        /// <param name="individual">Individual that has changed properties.</param>
        /// <returns>List with properties changes or empty list if there are no properties.</returns>
        ReadOnlyCollection<PropertyChangesData> GetProperties(Individual individual);
    }
}
