﻿//  ------------------------------------------------------------------------
//  <copyright file="ITriple.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------
namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Interface describes RDF format triple.
    /// <remarks>The RDF triple consists of three parts: subject, predicate and object. 
    /// The subject and object have the type (<see cref="PetrSU.PetrSU.SmartSlog.KpiWrapper.RdfType"/>).
    /// The subject can have type URI or blank node. Possible types for object are URI, blank node or literal.
    /// </remarks>
    /// </summary>
    /// <seealso cref="PetrSU.SmartSlog.KpiWrapper.RdfType"/>
    public interface ITriple
    {
        /// <summary>
        /// Gets or sets the triple's subject.
        /// </summary>
        /// <exception cref="System.ArgumentNullException">Thrown when the value is null.</exception>
        string Subject { get; set; }

        /// <summary>
        /// Gets or sets the triple's predicate.
        /// </summary>
        /// <exception cref="System.ArgumentNullException">Thrown when the value is null.</exception>
        string Predicate { get; set; }

        /// <summary>
        /// Gets or sets the triple's object.
        /// </summary>
        /// <exception cref="System.ArgumentNullException">Thrown when the value is null.</exception>
        string ObjectElement { get; set; }

        /// <summary>
        /// Gets or sets the type of the object.
        /// </summary>
        RdfType SubjectType { get; set; }

        /// <summary>
        /// Gets or sets the type of the subject.
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">Thrown when the value is literal.</exception>
        RdfType ObjectType { get; set; }

        /// <summary>
        /// Set subject and it's type. Type for subject can be URI or blank node.
        /// </summary>
        /// <param name="value">Subject value of the triple.</param>
        /// <param name="type">Type of the subject.</param>
        /// <exception cref="System.ArgumentNullException">Thrown when the subject is null.</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">Thrown when the value is literal.</exception>
        void SetSubject(string value, RdfType type);

        /// <summary>
        /// Set subject and it's type.
        /// </summary>
        /// <param name="value">Object value of the triple.</param>
        /// <param name="type">Type of the subject.</param>
        /// <exception cref="System.ArgumentNullException">Thrown when the subject is null.</exception>
        void SetObject(string value, RdfType type);
    }
}
