﻿// ------------------------------------------------------------------------
// <copyright file="KpiHelper.cs" company="PetrSU">
//  SmartSlog KP Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//
//  SmartSlog KP Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog KP Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
// </copyright>
// 
// <author>SmartSlog Team (Aleksandr A. Lomov)</author>
// <email>mailto:lomov@cs.karelia.ru</email>
// <created>21.01.2013 15:29:27</created>
// <lastedit>28.01.2013 14:29:22</lastedit>
// ------------------------------------------------------------------------

namespace PetrSU.SmartSlog
{
    using System;
    using System.Collections.Generic;
    using System.IO;

    internal static class KpiHelper
    {
        public delegate void kpiFunction(ICollection<ITriple> triples);

        public delegate ISubscriptionInfo kpiSubscribeFunction(ICollection<ITriple> triples);

        public delegate void kpiUnsubscribe(ISubscriptionInfo subscriptionInfo);

        public delegate SubscriptionStatus kpiCheckSubscription(ISubscriptionInfo subscriptionInfo, int timeout);

        public delegate void kpiUpdateFunction(ICollection<ITriple> oldTriples, ICollection<ITriple> newTriples);

        public delegate int kpiResultFunction(ICollection<ITriple> triples, ICollection<ITriple> resultTriples);

        public delegate void kpiEmptyFunction();

        public static void DoKpi(kpiEmptyFunction function, int managementType, string methodName)
        {
            try
            {
                function();
            }
            catch (IOException e)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, e);
                throw e; // IO
            }
            catch (SmartSpaceException e)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, e);
                throw e;
            }
            catch (KpiException e)
            {
                throw new InvalidOperationException("KPI failed: " + e.ToString());
            }
        }

        public static void DoKpi(kpiFunction function, ICollection<ITriple> triples, int managementType, string methodName)
        {
            try
            {
                function(triples);
            }
            catch (IOException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (SmartSpaceException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (KpiException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw new InvalidOperationException("KPI failed: " + ex.ToString());
            }
        }

        public static int DoKpi(kpiResultFunction function, ICollection<ITriple> triples, ICollection<ITriple> resultTriples, int managementType, string methodName)
        {
            try
            {
                return function(triples, resultTriples);
            }
            catch (IOException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (SmartSpaceException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (KpiException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw new InvalidOperationException("KPI failed: " + ex.ToString());
            }
        }

        public static void DoKpiUpdate(kpiUpdateFunction function, ICollection<ITriple> oldTriples, ICollection<ITriple> newTriples, int managementType, string methodName)
        {
            try
            {
                function(oldTriples, newTriples);
            }
            catch (IOException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (SmartSpaceException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw ex;
            }
            catch (KpiException ex)
            {
                LoggerManager.WriteExceptionMessage(managementType, methodName, ex);
                throw new InvalidOperationException("KPI failed: " + ex.ToString());
            }
        }


        public static void DoKpiUpdate(kpiUpdateFunction function, ICollection<ITriple> oldTriples, ICollection<ITriple> newTriples)
        {
            function(oldTriples, newTriples);
        }

        //public static void DoKpi(kpiFunction function, ICollection<ITriple> triples)
        //{
        //    try
        //    {
        //        function(triples);
        //    }
        //    catch (IOException ex)
        //    {
        //        throw ex;
        //    }
        //    catch (SmartSpaceException ex)
        //    {
        //        throw ex;
        //    }
        //    catch (KpiException ex)
        //    {
        //        throw new InvalidOperationException("KPI failed: " + ex.ToString());
        //    }
        //}

        //public static ISubscriptionInfo DoKpiSubscribe(kpiSubscribeFunction function, ICollection<ITriple> triples)
        //{
        //    try
        //    {
        //       return function(triples);
        //    }
        //    catch (IOException ex)
        //    {
        //        throw ex;
        //    }
        //    catch (SmartSpaceException ex)
        //    {
        //        throw ex;
        //    }
        //    catch (KpiException ex)
        //    {
        //        throw new InvalidOperationException("KPI failed: " + ex.ToString());
        //    }
        //}

        //public static void DoKpiUnsubscribe(kpiUnsubscribe function, ISubscriptionInfo subscriptionInfo)
        //{
        //    try
        //    {
        //        function(subscriptionInfo);
        //    }
        //    catch (IOException ex)
        //    {
        //        //throw ex;
        //    }
        //    catch (SmartSpaceException ex)
        //    {
        //        //throw ex;
        //    }
        //    catch (KpiException ex)
        //    {
        //        //throw new InvalidOperationException("KPI failed: " + ex.ToString());
        //    }
        //}


        //public static SubscriptionStatus DoKpiCheckSubscription(kpiCheckSubscription function, ISubscriptionInfo subscriptionInfo, int timeout)
        //{
        //    try
        //    {
        //        return function(subscriptionInfo, timeout);
        //    }
        //    catch (IOException ex)
        //    {
        //        //throw ex;
        //    }
        //    catch (SmartSpaceException ex)
        //    {
        //        //throw ex;
        //    }
        //    catch (KpiException ex)
        //    {
        //        //throw new InvalidOperationException("KPI failed: " + ex.ToString());
        //    }

        //    return null;//SubscriptionStatus.Error;
        //}



        public static void Do(IKpi kpi, ICollection<ITriple> oldTriples, ICollection<ITriple> newTriples)
        {

            if (newTriples == null || newTriples.Count == 0)
            {
                //KpiHelper.DoKpi(kpi.Remove, oldTriples);
            }
            else if (oldTriples == null || oldTriples.Count == 0)
            {
                //KpiHelper.DoKpi(kpi.Insert, newTriples);
            }
            else
            {
                //KpiHelper.DoKpiUpdate(kpi.Update, oldTriples, newTriples);
            }
        }


    }
}
