﻿//  ------------------------------------------------------------------------
//  <copyright file="NativeMethods.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace PetrSU.SmartSlog
{
    /// <summary>
    /// <c>NativeMethods</c> class contains description of functions for work with KPI_low library.
    /// <remarks>
    /// This is internal class, you can't use it. Use <see cref="KPILowLibWrapper.KPILowWrapper"/> class.
    /// </remarks>
    /// <seealso cref="KPILowLibWrapper.KPILowWrapper"/>
    /// </summary>
    internal sealed class NativeMethods
    {
        /// <summary>
        /// Path to the SmartSlog dll file (version 0.6 or higher).
        /// </summary>
        private const string smartslogPath = "smartslog.dll";

        /// <summary>
        /// Path to the Windows socket dll file.
        /// </summary>
        private const string windowsSoscksPath = "WS2_32.dll";

        /// <summary>
        /// Path to the pthread dll file.
        /// </summary>
        private const string pthreadPath = "pthreadVC2.dll";

        /// <summary>
        /// Path to the ckpi dll file.
        /// </summary>
        private const string ckpiPath = "ckpi.dll";

        private NativeMethods() { }

        [DllImport(ckpiPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr ss_discovery([MarshalAs(UnmanagedType.LPStr)] string broadcast_address, int port);

        #region Functions for operations with triples in smart space

        // Triples
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_triple([MarshalAs(UnmanagedType.LPStr)] string subject,
            [MarshalAs(UnmanagedType.LPStr)] string predicate,
            [MarshalAs(UnmanagedType.LPStr)] string obj,
            int rdf_subject_type,
            int rdf_object_type);

        // Triples
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_triple_detached([MarshalAs(UnmanagedType.LPStr)] string subject,
            [MarshalAs(UnmanagedType.LPStr)] string predicate,
            [MarshalAs(UnmanagedType.LPStr)] string obj,
            int rdf_subject_type,
            int rdf_object_type);



        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_insert_triple(IntPtr node, IntPtr triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_insert_triples(IntPtr node, IntPtr triples);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_remove_triple(IntPtr node, IntPtr triple_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_remove_triples(IntPtr node, IntPtr triple_templates);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        // Return list_t*
        public static extern IntPtr sslog_node_query_triple(IntPtr node, IntPtr triple_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        // Return list_t*
        public static extern IntPtr sslog_node_query_triples(IntPtr node, IntPtr triple_templates);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_update_triple(IntPtr node, IntPtr current_triple, IntPtr new_triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_update_triples(IntPtr node, IntPtr current_triples, IntPtr new_triples);

        #endregion

        #region Node functions for SPARQL queries

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        // Return sslog_sparql_result_t*
        public static extern IntPtr sslog_node_sparql_enpoint_select(
            IntPtr node,
            [MarshalAs(UnmanagedType.LPStr)] string endpoint_address,
            [MarshalAs(UnmanagedType.LPStr)] string query,
            [MarshalAs(UnmanagedType.LPStr)] string extra_parameters,
            [MarshalAs(UnmanagedType.LPStr)] string triples_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_sparql_ask(IntPtr node, [MarshalAs(UnmanagedType.LPStr)] string query);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        // Return sslog_sparql_result_t*
        public static extern IntPtr sslog_node_sparql_select(
            IntPtr node,
            [MarshalAs(UnmanagedType.LPStr)] string query,
            [MarshalAs(UnmanagedType.LPStr)] string triples_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        // Return list_t*
        public static extern IntPtr sslog_node_sparql_construct(IntPtr node, [MarshalAs(UnmanagedType.LPStr)] string query);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_free_sparql_result(IntPtr result);

        #endregion

        #region Functions for operations in local store

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_insert_triple(IntPtr triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_insert_triples(IntPtr triples);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_remove_triple(IntPtr triple_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_remove_triples(IntPtr triple_templates);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        // Return list_t
        public static extern IntPtr sslog_query_triple(IntPtr triple_template);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        // Return list_t
        public static extern IntPtr sslog_query_triples(IntPtr triples_templates);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_update_triple(IntPtr current_triple, IntPtr new_triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_update_triples(IntPtr current_triples, IntPtr new_triples);

        #endregion

        #region TripleProperty

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl,
            EntryPoint = "sslog_triple_get_subject")]
        public static extern IntPtr sslog_triple_get_subject_ptr(IntPtr triple);

        public static string sslog_triple_get_subject(IntPtr triple)
        {
            IntPtr ptr = sslog_triple_get_subject_ptr(triple);
            return Marshal.PtrToStringAnsi(ptr);
        }

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl,
            EntryPoint = "sslog_triple_get_predicate")]
        public static extern IntPtr sslog_triple_get_predicate_ptr(IntPtr triple);

        public static string sslog_triple_get_predicate(IntPtr triple)
        {
            IntPtr ptr = sslog_triple_get_predicate_ptr(triple);
            return Marshal.PtrToStringAnsi(ptr);
        }

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl,
            EntryPoint = "sslog_triple_get_object")]
        public static extern IntPtr sslog_triple_get_object_ptr(IntPtr triple);

        public static string sslog_triple_get_object(IntPtr triple)
        {
            IntPtr ptr = sslog_triple_get_object_ptr(triple);
            return Marshal.PtrToStringAnsi(ptr);
        }

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern RdfType sslog_triple_get_object_type(IntPtr triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern RdfType sslog_triple_get_subject_type(IntPtr triple);


        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern bool sslog_triple_is_stored(IntPtr triple);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_free_triple(IntPtr triple);

        #endregion

        #region Subscription functions (subscription.h)

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        // Return: sslog_subscription_t*
        public static extern IntPtr sslog_new_subscription(IntPtr node, bool is_asynchronous);

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_free_subscription(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_is_active(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl, EntryPoint = "sslog_sbcr_get_id")]
        // Return: const char*
        public static extern IntPtr sslog_sbcr_get_id_ptr(IntPtr subscription);

        public static string sslog_sbcr_get_id(IntPtr triple)
        {
            IntPtr ptr = sslog_sbcr_get_id_ptr(triple);
            return Marshal.PtrToStringAnsi(ptr);
        }

        #region TripleSparqlChanges

        [DllImport(smartslogPath, SetLastError = true,
        CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_get_changes_last(IntPtr subscription);
        // Return: sslog_sbcr_changes_t *

        [DllImport(smartslogPath, SetLastError = true,
        CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_ch_get_new_sparql_select(IntPtr changes);

        [DllImport(smartslogPath, SetLastError = true,
        CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_ch_get_old_sparql_select(IntPtr changes);

        [DllImport(smartslogPath, SetLastError = true,
        CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_ch_get_triples(IntPtr changes, int action);
        // Return: list_t* 
        #endregion

        #region Individual Changes

        /**
          * * @brief Gets a suquence number of the changes.
          * *
          * * Changes are made based on data that were received with notification.
          * * This number is sequence of indication that was used to make changes.
          * * Firts changes are made while perfoming subscription operation,
          * * this changes has '1' as this number.
          * *
          * * This function sets a global error if it can't perfome operation,
          * * you can get an error code using #sslog_get_error_code function.
          * *
          * * @param[in] changes subscription chnages data structure.
          * *
          * @return sequence number of the chnages.
          */
        //SSLOG_EXTERN int sslog_sbcr_ch_get_sequence_number(sslog_sbcr_changes_t *changes);
        // Not added
        [DllImport(smartslogPath, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_ch_get_sequence_number(IntPtr changes);


        /**
         * @brief Gets all individual's URIs by action.
         *
         * Individuals can be inserted, removed or updated. Updated means that some
         * properties of the individual were changed.
         *
         * <b>Do not modify or free returned list. </b>
         *
         * This function sets a global error if it can't perfome operation,
         * you can get an error code using #sslog_get_error_code function.
         *
         * @param[in] changes. Subscription chnages data to get information.
         * @param[in] action. Action that was perfom under individuals: insert, remove,
         * update.
         *
         * @return list with individuals URIs (list with strings) or
         * NULL if given changes or action is incorrect.
         */
        //SSLOG_EXTERN const list_t *sslog_sbcr_ch_get_individual_by_action(sslog_sbcr_changes_t *changes, sslog_action_type action);
        // Test
        [DllImport(smartslogPath, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_ch_get_individual_by_action(IntPtr changes, int action);

        /**
         * @brief Gets all individual's UUID.
         *
         * It make one list with all individual's UUIDs, that are were inserted,
         * updated or removed.
         *
         * <b>You need to free the returned list, but without data relesed.</b>
         * Use #list_free_with_nodes with NULL as argument for free function.
         *
         * <b>Known bug:</b> if under the individual was perfomed more that one actions,
         * for example it was inserted and updated, then UUID will be inserted
         * to list more that once. This will be fixed later.
         *
         * This function sets a global error if it can't perfome operation,
         * you can get an error code using #sslog_get_error_code function.
         *
         * @param[in] changes. Changes to get individuals UUIDs.
         *
         * @return list with UUIDs (list with strings) on success or NULL otherwise.
         */
        //SSLOG_EXTERN list_t *sslog_sbcr_ch_get_individual_all(sslog_sbcr_changes_t *changes);
        // Not added
        [DllImport(smartslogPath, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_sbcr_ch_get_individual_all(IntPtr changes);

        #endregion

        #region SubcriptionManagment

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_sbcr_set_node(IntPtr subscription, IntPtr node);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_add_individual(IntPtr subscription, IntPtr individual, /* list_t* */ IntPtr properties);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_add_class(IntPtr subscription, IntPtr ont_class);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_add_triple_template(IntPtr subscription, IntPtr triple_template);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_add_triples_templates(IntPtr subscription, /* list_t* */ IntPtr triples_templates);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_add_sparql_select(IntPtr subscription,
            [MarshalAs(UnmanagedType.LPStr)] string select_query,
            [MarshalAs(UnmanagedType.LPStr)] string select_triples_templates);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_subscribe(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_unsubscribe(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_sbcr_unsubscribe_all(IntPtr node, bool with_free);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_wait(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_sbcr_stop(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_sbcr_stop_all();

        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        internal delegate void SubscriptionHandler(IntPtr subscription);

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void sslog_sbcr_set_changed_handler(IntPtr subscription, SubscriptionHandler handler);

        #endregion

        #endregion

        #region List function

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr list_new();

        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        public static extern void list_add_data(IntPtr list, IntPtr data);



        [DllImport(smartslogPath, SetLastError = true,
            CallingConvention = CallingConvention.Cdecl)]
        // list_t *list_get_next_node(list_t *list, list_t *previous_node, void **out_data)
        public static extern IntPtr list_get_next_node(IntPtr list, IntPtr previous_node, ref IntPtr out_data);

        public static Collection<ITriple> ToManagedList(IntPtr list)
        {
            IntPtr data = IntPtr.Zero;
            IntPtr firsrtNode = list_get_next_node(list, IntPtr.Zero, ref data);
            IntPtr previousNode = firsrtNode;

            Collection<ITriple> triples = new Collection<ITriple>();

            if (previousNode == IntPtr.Zero)
            {
                return triples;
            }

            do
            {
                triples.Add(Triple.Create(data));

                previousNode = list_get_next_node(list, previousNode, ref data);

            } while (previousNode != firsrtNode);

            return triples;
        }


        #endregion

        #region Individual functions

        #region Create (individual, property, class)
        /**
         * @brief Creates an individual.
         * Try to find information about individual in the local store by given URI and class.
         * If the indormation is founded then an individual will be returned.
         * If the is not informations then a new triple created and based on it
         * created a new individual.
         *
         * Each class are represented in the local store with triple:
         * URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - Class-URI
         *
         * Function sets information about errors (#errors.h).
         *
         * @param parent_class. Parent class to create an individual.
         * @param uri. URI of the individual.
         * @return class on success or NULL otherwise (error or no information in the local store).
         */
        //SSLOG_EXTERN sslog_individual_t *sslog_new_individual(const sslog_class_t *parent_class, const char *uri);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_individual(IntPtr parent_class, [MarshalAs(UnmanagedType.LPStr)] string uri);


        /**
 * @brief Cheks is a property has an object type.
 * @param property. Property to check type.
 * @return true if the property is object or false otherwise (property has a data type or property is NULL).
 */
        //SSLOG_EXTERN bool sslog_property_is_object(sslog_property_t *property);
        // TEST IT!!!
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool sslog_property_is_object(IntPtr property);

        /**
 *
 * @brief Creates a class.
 * Try to find information about class in the local store by given URI and returns
 * founded class. If the is not informations then a new triple created and based on it
 * created a new class.
 *
 * Each class are represented in the local store with triple:
 * URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/2000/01/rdf-schema#Class
 *
 * Function sets information about errors (#errors.h).
 *
 * @param uri. URI of the class.
 * @return class on success or NULL otherwise (error or no information in the local store).
 */
        //sslog_class_t* sslog_new_class(const char *uri);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_class([MarshalAs(UnmanagedType.LPStr)] string uri);

        /**
         * @brief Creates a property.
         * Try to find information about property in the local store by given URI and returns
         * founded property. If the is not informations then a new triple created and based on it
         * created a new property.
         *
         * Each class are represented in the local store with triple:
         * URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/1999/02/22-rdf-syntax-ns#Property
         *
         * Function sets information about errors (#errors.h).
         *
         * @param uri. URI of the property.
         * @return class on success or NULL otherwise (error or no information in the local store).
         */
        //SSLOG_EXTERN sslog_property_t *sslog_new_property(const char *uri, sslog_property_type type);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_property([MarshalAs(UnmanagedType.LPStr)] string uri, int type);

        #endregion

        #region SS_action

        // Include later
        #region Triples
        //            /**
        /// @brief Gets an entity (class, property, individual) from triple.
        ///
        /// If the triple describes a class (http://www.w3.org/2000/01/rdf-schema#Class)
        /// then class structure will be returned.
        ///
        /// If the triple describes a property (http://www.w3.org/1999/02/22-rdf-syntax-ns#Property)
        /// then property structure will be returned. 
        ///
        /// If the triple describes an individual (URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - classURI)
        /// then the individual structure will be returned.
        ///
        /// If the triple describes a property value of some individuals (URI - propertyURI - URI/Data)
        /// then the owner (individual) of this property will be returned. In this way the individual
        /// can be returned if there is a rdf:type triple for the individual with 'URI'.
        ///
        /// @param triple. Triple to convert.
        /// @return entity structure on success or NULL otherwise (incorrect triple).
        ////
        //SSLOG_EXTERN sslog_entity_t *sslog_entity_from_triple(sslog_triple_t *triple);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_entity_from_triple(IntPtr triple);


        ///**
        /// @brief Gets individual from triple.
        ///
        /// If the triple describes an individual (URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - classURI)
        /// then the individual structure will be returned.
        ///
        /// If the triple describes a property value of some individuals (URI - propertyURI - URI/Data)
        /// then the owner (individual) of this property will be returned. In this way the individual
        /// can be returned if there is a rdf:type triple for the individual with 'URI'.
        ///
        /// @param triple. Triple to convert.
        /// @return individual on success or NULL otherwise (triple not describes individual, no rdf:type for individual).
        ////
        //SSLOG_EXTERN sslog_individual_t* sslog_individual_from_triple(sslog_triple_t *triple);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_individual_from_triple(IntPtr triple);

        ///**
        /// @brief Gets class from triple.
        ///
        /// If the triple describes a class
        /// (ClassURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/2000/01/rdf-schema#Class)
        /// or individual (IndividualURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - classURI)
        /// then the property structure will be returned.
        ///
        /// @param triple. Triple to convert.
        /// @return class on success or NULL otherwise (triple not describes class).
        ////
        //SSLOG_EXTERN sslog_class_t *sslog_class_from_triple(sslog_triple_t *triple);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_class_from_triple(IntPtr triple);

        ///
        /// @brief Gets property from triple.
        ///
        /// If the triple describes a property
        /// (PropertyURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/1999/02/22-rdf-syntax-ns#Property)
        /// or property value (IndividualURI - PropertyURI - Object/Data)
        /// then the property structure will be returned.
        ///
        /// @param triple. Triple to convert.
        /// @return property on success or NULL otherwise (triple not describes property).
        //SSLOG_EXTERN sslog_property_t *sslog_property_from_triple(sslog_triple_t *triple);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_property_from_triple(IntPtr triple);

        ///**
        /// @brief Gets rdf:type triple of entity (class, property, individual).
        ///
        /// If the entity is a class then class rdf:type triple
        /// (ClassURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/2000/01/rdf-schema#Class)
        /// will be returned.
        ///
        /// If the entity is a property then property rdf:type triple
        /// (PropertyURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/1999/02/22-rdf-syntax-ns#Property)
        /// will be returned.
        ///
        /// If the entity is an individual then individual rdf:type triple
        /// (IndividualURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - ClassURI)
        /// will be returned.
        ///
        /// @param triple. Triple to convert.
        /// @return entity structure on success or NULL otherwise (incorrect entity).
        ////
        //SSLOG_EXTERN sslog_triple_t* sslog_entity_to_triple(sslog_entity_t *entity);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_entity_to_triple(IntPtr entity);

        ///**
        /// @brief Gets rdf:type triple of individual.
        ///
        /// Individual is described with rdf:type triple (URI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - classURI)).
        ///
        /// @param individual. Individual to convert.
        /// @return triple on success or NULL otherwise (individual is NULL).
        ////
        //SSLOG_EXTERN sslog_triple_t * sslog_individual_to_triple(sslog_individual_t *individual);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_individual_to_triple(IntPtr individual);

        ///**
        /// @brief Gets rdf:type triple of property.
        /// Rdf:type triple of property:
        /// PropertyURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/1999/02/22-rdf-syntax-ns#Property.
        /// @param individual. Property to convert.
        /// @return triple on success or NULL otherwise (property is NULL).
        ////
        //SSLOG_EXTERN sslog_triple_t * sslog_property_to_triple(sslog_property_t *property);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_property_to_triple(IntPtr property);

        ///**
        /// @brief Gets rdf:type triple of class.
        ///
        /// Rdf:type triple of class:
        /// ClassURI - http://www.w3.org/1999/02/22-rdf-syntax-ns#type - http://www.w3.org/2000/01/rdf-schema#Class.
        ///
        /// @param individual. Property to convert.
        /// @return triple on success or NULL otherwise (property is NULL).
        ////
        //SSLOG_EXTERN sslog_triple_t * sslog_class_to_triple(sslog_class_t *ont_class);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_class_to_triple(IntPtr ont_class);

        ///** @} */ // End of Entity-triple-converting group.
        ///*****************************************************************************/

        #endregion

        // !!! DONE !!!
        #region IndividualPropertyLocal
        ///******************* Functions for individual properties *********************/
        ///**
        /// @defgroup Individual-local-property Local functions for individual properties
        /// @ingroup Individual
        /// Functions are used to local opeation (in local store) with individual properties.
        /// @{
        ////

        ///**
        /// @brief Gets individial property value (data or object).
        /// If the property has an data-type, then the string will be returned,
        /// if the property has an object-type, then an individual will be returned.
        /// If the individual has several propertirs for given property when a first founded value
        /// will be returned.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to get property value.
        /// @param property. Property to get value.
        /// @return string (char *) for data-property or individual for object-property on success and
        /// NULL otherwise (error or value not found).
        ////
        //SSLOG_EXTERN const void *sslog_get_property(sslog_individual_t *individual, sslog_property_t *property);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_get_property(IntPtr individual, IntPtr property);

        ///**
        /// @brief Gets all values for individual property.
        /// If the property has an data-type, then the strings will be returned,
        /// if the property has an object-type, then an individuals will be returned.
        ///
        /// Returned list MUST be deleted without deleting datas (use #list_free_with_nodes).
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to get property value.
        /// @param property. Property to get value.
        /// @return list with strings (char *) for data-property or list with individuals for object-property on success,
        /// empty list when there are no values for property and NULL otherwise (error).
        ////
        //SSLOG_EXTERN list_t *sslog_get_properties(sslog_individual_t *individual, sslog_property_t *property);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_get_properties(IntPtr individual, IntPtr property);

        ///**
        /// @brief Adds a property for individual.
        /// Property can be data or object, for data property value must be a string (char *),
        /// for object proeprty value must be an individual. If the inividual has a given property
        /// with the same value, then function does nothing.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to add a property.
        /// @param property. Property to add.
        /// @param value. Value of property. String for data-property, individual for object-property.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_insert_property(sslog_individual_t *individual, sslog_property_t *property, void *value);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_insert_property(IntPtr individual, IntPtr property, IntPtr value);

        ///**
        /// @brief Removes value of individual property.
        /// Removes data or object property with the given value.
        /// For data-property value is a string (char *), for object-property is an individual.
        /// If the inividual hasn't a property with the given value, then function does nothing.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to remove a property.
        /// @param property. Property to remove.
        /// @param value. Value of property. String for data-property, individual for object-property.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_remove_property(sslog_individual_t *individual, sslog_property_t *property, void *value);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_remove_property(IntPtr individual, IntPtr property, IntPtr value);

        ///**
        /// @brief Removes all values if individual property.
        /// Removes all values of data or object property.
        /// If the inividual hasn't a given property values, then function does nothing.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to remove a property values.
        /// @param property. Property to remove.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_remove_properties(sslog_individual_t *individual, sslog_property_t *property);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_remove_properties(IntPtr individual, IntPtr property);

        ///**
        /// @brief Updates an individual property value.
        /// For data-property values are a strings (char *), for object-property are individuals.
        /// If the current_value is NULL, then function works as #sslog_add_property.
        /// If the new_value is NULL, then function works as #sslog_remove_property.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to update property value.
        /// @param property. Property to update.
        /// @param current_value. Current value of the property.
        /// @param new_value. New value of the property.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_update_property(sslog_individual_t *individual, sslog_property_t *property, void *current_value, void *new_value);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_update_property(IntPtr individual, IntPtr property, IntPtr current_value, IntPtr new_value);

        ///** @} */ // End of Individual-local-property group.
        ///*****************************************************************************/

        #endregion

        // !!! DONE !!!
        #region IndividualPropertyNode

        ///**************** Node functions for individual properties *******************/
        ///**
        /// @ingroup Individual
        /// @defgroup Individual-node-property Node functions for individual properties
        /// Functions are used to remote opeations (in smart space) with individual properties.
        /// @{
        ////

        ///**
        /// @brief Gets one property from the smart space and inserts it to local store.
        /// Function gets only one first founded property. Founded property (  triple ) will be stored too.
        /// If the object property is getting,   then the rdf:type triple for individual (  property value )
        /// must be in the local store or in the smart space,   otherwise individual can't be created and
        /// function returns NULL.
        ///
        /// Function sets an information about errors (  see #errors.h ).
        ///
        /// @param[in] node. Node to work with the smart space.
        /// @param[in] individual. Individual to get property.
        /// @param[in] property. Property to get (  object ot data ).
        /// @return value of property (string for data-property/individual for object) on success or NULL otherwise (error or not found).
        ////
        //SSLOG_EXTERN const void* sslog_node_get_property(sslog_node_t *node, sslog_individual_t *individual, sslog_property_t *property);
        // Added
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_node_get_property(IntPtr node, IntPtr individual, IntPtr property);

        ///**
        /// @brief Inserts property to the smart space and to the local store.
        /// Function converts property to triple and inserts it into smart space and store triple in the local store.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param node. Node to work with the smart space.
        /// @param individual. Individual to set property.
        /// @param property. Property of individual to set.
        /// @param value. Value of the property, can be a string (for data-properties) or individual (for object-properties).
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_insert_property(sslog_node_t *node, sslog_individual_t *individual, sslog_property_t *property,
        //                                            void *value);
        // TEST
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_insert_property(IntPtr node, IntPtr individual, IntPtr property, IntPtr value);

        ///**
        /// @brief Removes property from the smart space nd from local store.
        /// Function converts property to triple and removes it from smart space and from the local store.
        /// If the value of property is NULL then all such properties will be removed.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param node. Node to work with the smart space.
        /// @param individual. Individual to remove property.
        /// @param property. Property of individual to remove.
        /// @param value. Value of the property,
        /// can be a string (for data-properties), individual (for object-properties)
        /// or NULL for removing all such properties.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_remove_property(sslog_node_t *node, sslog_individual_t *individual, sslog_property_t *property,
        //                                            void *value);
        // TEST
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_remove_property(IntPtr node, IntPtr individual, IntPtr property, IntPtr value);

        ///**
        /// @brief Updates property in the smart space and in the local store.
        /// Function converts property to triple and updates value from current to new in the smart space and in the local store.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param node. Node to work with the smart space.
        /// @param individual. Individual to update property.
        /// @param property. Property to update.
        /// @param current_value. Current value of the property, this value will be removed.
        /// @param new_value. New value of the property, this value will be inserted to the smart space and added to local store.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_update_property(sslog_node_t *node, sslog_individual_t *individual, sslog_property_t *property,
        //                                            void *current_value, void *new_value);
        // TEST
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_update_property(IntPtr node, IntPtr individual, IntPtr property, IntPtr current_value, IntPtr new_value);

        ///** @} */ // End of Individual-node-property group.
        ///*****************************************************************************/

        #endregion

        // !!! DONE !!!
        #region IndividualWholeLocal

        ///********************** Functions for whole individual ***********************/
        ///**
        /// @defgroup Individual-local-whole Local functions for whole individual
        /// @ingroup Individual
        /// Functions are used to local opeations (in local store) with whole individual.
        /// @{
        ////

        ///**
        /// @brief Gets an individual by URI.
        /// @param[in] uri. URI of the individual.
        /// @return individual on success or NULL otherwise (error, no triple that describes individual).
        ////
        //SSLOG_EXTERN sslog_individual_t *sslog_get_individual(const char *uri);
        // Test! NOT WORKING
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_get_individual([MarshalAs(UnmanagedType.LPStr)] string value);


        ///**
        /// @brief Removes individual from the local store.
        /// Functiosn converts individual to triples and removes it.
        /// Individual structure CANNOT BE used any more.
        ///
        /// Function sets information about errors (#errors.h).
        ///
        /// @param individual. Individual to remove.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_remove_individual(sslog_individual_t *individual);
        // Test
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_remove_individual(IntPtr individual);

        ///** @} */ // End of Individual-local-whole group.
        ///*****************************************************************************/

        #endregion

        // !!! DONE !!!
        #region IndividualWholeNode

        ///******************** Node functions for whole individual ********************/
        ///**
        /// @defgroup Individual-node-whole Node functions for whole individual
        /// @ingroup Individual
        /// Functions are used to remote opeations (in smart space) with whole individual.
        /// @{
        ////

        ///**
        /// @brief Get all individual by uri.
        /// Function get individual with given URI (rdf:type triple with object equals URI).
        ///
        /// Function sets an information about errors (see #errors.h).*
        ///
        /// @param node. Node to work with the smart space.
        /// @param uri. URI of the individual.
        /// @return individual on success or NULL otherwise (error or not found).
        ////
        //SSLOG_EXTERN sslog_individual_t * sslog_node_get_individual_by_uri(sslog_node_t *node, const char *uri);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_node_get_individual_by_uri(IntPtr node, [MarshalAs(UnmanagedType.LPStr)] string uri);

        ///**
        /// @brief Get all individual by class.
        /// Function get all individuals with rdf:type where object equals class URI.
        ///
        /// Function sets an information about errors (see #errors.h).*
        ///
        /// @param node. Node to work with the smart space.
        /// @param ont_class. Base class of individuals.
        /// @return list with individuals on success or NULL otherwise.
        ////
        //SSLOG_EXTERN list_t* sslog_node_get_individuals_by_class(sslog_node_t *node, sslog_class_t *ont_class);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_node_get_individuals_by_class(IntPtr node, IntPtr ont_class);

        ///**
        /// @brief Gets all properties about individual.
        /// The function gets triples that are represent an individual and properties.
        /// If the individual has object properties
        /// (links to other individuals) then only rdf:type triple of linked individual
        /// will be queried from the smart space.
        /// All triples will be stored locally.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param[in] node. Node to work with the smart space.
        /// @param[in] individual. Individual to populate.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_populate(sslog_node_t *node, sslog_individual_t *individual);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_populate(IntPtr node, IntPtr individual);

        ///**
        /// @brief Inserts the individual to the smart space.
        /// The function gets triples that are represent an individual and properties and
        /// inserts it to the smart space. If the individual has object properties
        /// (links to other individuals) then only rdf:type triple of linked individual
        /// will be inserted.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param[in] node. Node to work with the smart space.
        /// @param[in] individual. Individual to insert.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_insert_individual(sslog_node_t *node, sslog_individual_t *individual);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_insert_individual(IntPtr node, IntPtr individual);

        ///**
        /// @brief Removes the individual from the smart space and all properties from local store.
        /// The function gets triples that are represent an individual and properties and
        /// removes it from the smart space.
        /// This function does not remove rdf:type triple (i.e. individual)
        /// from the local store and individual structure can be used after calling function.
        /// To fully remove individual from the smart space and local store use
        /// #sslog_node_remove_individual_with_local function.
        ///
        /// Linked individuals with object properties will not be removed from the smart space.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param[in] node. Node to work with the smart space.
        /// @param[in] individual. Individual to insert.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ///
        /// @see sslog_node_remove_individual_with_local
        ////
        //SSLOG_EXTERN int sslog_node_remove_individual(sslog_node_t *node, sslog_individual_t *individual);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_remove_individual(IntPtr node, IntPtr individual);

        ///**
        /// @brief Removes the individual from the smart space and from local store.
        /// The function gets triples that are represent an individual and properties and
        /// removes it from the smart space and local store.
        /// Individual structure CANNOT BE used any more.
        ///
        /// Linked individuals with object properties will not be removed from the smart space.
        ///
        /// Function sets an information about errors (see #errors.h).
        ///
        /// @param[in] node. Node to work with the smart space.
        /// @param[in] individual. Individual to insert.
        /// @return SSLOG_ERROR_NO on success or error code otherwise.
        ////
        //SSLOG_EXTERN int sslog_node_remove_individual_with_local(sslog_node_t *node, sslog_individual_t *individual);
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_remove_individual_with_local(IntPtr node, IntPtr individual);


        #endregion

        public static Collection<Individual> IndividualListToManagedList(IntPtr list, Session session = null)
        {
            IntPtr data = IntPtr.Zero;
            IntPtr firsrtNode = list_get_next_node(list, IntPtr.Zero, ref data);
            IntPtr previousNode = firsrtNode;

            Collection<Individual> individuals = new Collection<Individual>();

            if (previousNode == IntPtr.Zero)
            {
                return individuals;
            }

            do
            {
                individuals.Add(Individual.Create(data, session));

                previousNode = list_get_next_node(list, previousNode, ref data);

            } while (previousNode != firsrtNode);

            return individuals;
        }

        #endregion

        #endregion

        #region Erros: public function (errors.h)

        [StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential)]
        public struct sslog_node_s
        {

            /// char*
            [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPStr)]
            public string id;
        }


        /// Return Type: int
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_error_get_last_code();


        /// Return Type: char*
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl,
            EntryPoint = "sslog_error_get_last_text")]
        //[return: MarshalAs(UnmanagedType.AnsiBStr)] - does't work, return IntPtr and marshal it to string.
        public static extern IntPtr sslog_error_get_last_text_ptr();

        public static string sslog_error_get_last_text()
        {
            IntPtr ptr = sslog_error_get_last_text_ptr();
            return Marshal.PtrToStringAnsi(ptr);
        }
        /// Return Type: int
        ///node: sslog_node_t*
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_error_get_code(ref sslog_node_s node);


        /// Return Type: char*
        ///node: sslog_node_t*
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern System.IntPtr sslog_error_get_text(ref sslog_node_s node);


        #endregion

        #region Node Functions
        /// Return Type: sslog_node_t*
        ///id: char*
        ///ss_id: char*
        ///address: char*
        ///port: int
        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr sslog_new_node(
            [In()] 
            [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPStr)] string id,
            [In()] 
            [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPStr)] string ss_id,
            [In()] 
            [MarshalAs(System.Runtime.InteropServices.UnmanagedType.LPStr)] string address,
            int port);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_join(IntPtr node);

        [DllImport(smartslogPath, CharSet = CharSet.Ansi, SetLastError = true, CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_node_leave(IntPtr node);

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_init();

        [DllImport(smartslogPath, SetLastError = true,
    CallingConvention = CallingConvention.Cdecl)]
        public static extern int sslog_shutdown();

        #endregion
    }
}
