﻿//  ------------------------------------------------------------------------
//  <copyright file="MarshalHelp.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>28.02.2016 20:54</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace PetrSU.SmartSlog.NativeStructures
{
    static internal class MarshalHelp
    {
        private static List<IntPtr> pointerList = new List<IntPtr>();

        public static string PtrToString(IntPtr p)
        {
            // TODO: deal with character set issues.  Will PtrToStringAnsi always
            // "Do The Right Thing"?
            if (p == IntPtr.Zero)
                return null;
            
            return Marshal.PtrToStringAnsi(p);
        }

        public static IntPtr StringToPtr(string original_string)
        {
            if (original_string == null)
                return IntPtr.Zero;

            IntPtr pointer = Marshal.StringToHGlobalAnsi(original_string);

            pointerList.Add(pointer);

            return pointer;
        }

        /// <summary>
        /// Make memory free from used string in unmanaged code
        /// </summary>
        public static void FreeIntPtr()
        {
            pointerList.ForEach(x => Marshal.FreeHGlobal(x));
            pointerList.Clear();
        }

        public static List<string> PtrToStringArray(IntPtr stringArray)
        {
            if (stringArray == IntPtr.Zero)
                return null;


            int argc = CountStrings(stringArray);
            return PtrToStringArray(argc, stringArray);
        }

        private static int CountStrings(IntPtr stringArray)
        {
            int count = 0;
            while (Marshal.ReadIntPtr(stringArray, count * IntPtr.Size) != IntPtr.Zero)
                ++count;
            return count;
        }

        public static int CountList(IntPtr list)
        {
            int count = 0;
            while (Marshal.ReadIntPtr(list, count * IntPtr.Size) != IntPtr.Zero)
                ++count;
            return count;
        }


        public static List<string> PtrToStringArray(int count, IntPtr stringArray)
        {
            if (count < 0)
                throw new ArgumentOutOfRangeException("count", "< 0");
            if (stringArray == IntPtr.Zero)
                return null;


            List<string> members = new List<string>();
            for (int i = 0; i < count; ++i)
            {
                IntPtr s = Marshal.ReadIntPtr(stringArray, i * IntPtr.Size);
                members.Add(PtrToString(s));
            }


            return members;
        }

        public static List<int> PtrToIntArray(IntPtr intArray)
        {
            if (intArray == IntPtr.Zero)
                return null;
            
            int count = 0;
            while (Marshal.ReadIntPtr(intArray, count * IntPtr.Size) != IntPtr.Zero)
                ++count;

            int[] tmp = new int[count];
            Marshal.Copy(intArray, tmp, 0, count);

            List<int> members = new List<int>(tmp);

            return members;
        }

        public static Collection<Individual> IndividualListToManagedList(IntPtr list)
        {
            IntPtr data = IntPtr.Zero;
            IntPtr firsrtNode = NativeMethods.list_get_next_node(list, IntPtr.Zero, ref data);
            IntPtr previousNode = firsrtNode;

            Collection<Individual> individuals = new Collection<Individual>();

            if (previousNode == IntPtr.Zero)
            {
                return individuals;
            }

            do
            {
                individuals.Add(Individual.Create(data));

                previousNode = NativeMethods.list_get_next_node(list, previousNode, ref data);

            } while (previousNode != firsrtNode);

            return individuals;
        }

        public static List<string> UriListToManagedList(IntPtr list)
        {
            IntPtr data = IntPtr.Zero;
            IntPtr firsrtNode = NativeMethods.list_get_next_node(list, IntPtr.Zero, ref data);
            IntPtr previousNode = firsrtNode;

            List<string> uriList = new List<string>();

            if (previousNode == IntPtr.Zero)
            {
                return uriList;
            }

            do
            {
                uriList.Add(PtrToString(data));

                previousNode = NativeMethods.list_get_next_node(list, previousNode, ref data);

            } while (previousNode != firsrtNode);

            return uriList;
        }


        public static IntPtr ToNativeList(List<Triple> triples)
        {
            IntPtr list = NativeMethods.list_new();

            foreach (Triple triple in triples)
            {
                NativeMethods.list_add_data(list, triple.NativePtr);
            }

            return list;
        }

        public static IntPtr ToNativeList(List<Property> properties)
        {
            IntPtr list = NativeMethods.list_new();

            foreach (Property property in properties)
            {
                NativeMethods.list_add_data(list, property.NativePtr);
            }

            //pointerList.Add(list);

            return list;
        }


    }
}
