﻿//  ------------------------------------------------------------------------
//  <copyright file="OntologyEntity.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Linq;

namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Class for representing ontology entities.
    /// </summary>
    public abstract class OntologyEntity : IEquatable<OntologyEntity>
    {
        /// <summary>
        /// Name of the entity.
        /// </summary>
        private string name;

        /// <summary>
        /// Namespace of the entity.
        /// </summary>
        private string @namespace;

        /// <summary>
        /// Full Uri of the entity.
        /// </summary>
        private string uri;

        /// <summary>
        /// Gets the name of the entity.
        /// </summary>
        public string Name
        {
            get { return this.name; }
        }

        /// <summary>
        /// Gets the namespace of the entity.
        /// </summary>
        public string Namespace
        {
            get { return this.@namespace; }
        }

        /// <summary>
        /// Gets Uri of the entity.
        /// </summary>
        public string Uri
        {
            get { return this.uri; }
        }

         /// <summary>
        /// Determines whether the specified object is equal to the current object. 
        /// Entities with the same Uri are equal.
        /// </summary>
        /// <param name="obj">Object to check.</param>
        /// <returns>true if the specified object is equal to the current object; otherwise, false.</returns>
        public override bool Equals(object obj)
        {
            return this.Equals(obj as OntologyEntity);
        }

        /// <summary>
        /// Determines whether the specified object is equal to the current object. 
        /// Entities with the same Uri are equal.
        /// </summary>
        /// <param name="other">The object to compare with the current object.</param>
        /// <returns>true if the specified object is equal to the current object; otherwise, false.</returns>
        public virtual bool Equals(OntologyEntity other)
        {
            if (other == null)
            {
                return false;
            }

            return this.Uri.Equals(other.Uri);
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>The hash code.</returns>
        public override int GetHashCode()
        {
            return this.Name.First().GetHashCode() + this.Uri.Length;
        }

        /// <summary>
        /// Returns the System.String that represents the entity.
        /// </summary>
        /// <returns>URI that represents the entity.</returns>
        public override string ToString()
        {
            return this.Uri;
        }

        /// <summary>
        /// Pars URI to the namespace and the name of the entity and sets necessary variables.
        /// </summary>
        /// <param name="uri">URI of the entity.</param>
        protected void SetUri(string uri)
        {
            int splitPosition = uri.LastIndexOf('#');

            if (splitPosition == -1)
            {
                splitPosition = uri.LastIndexOf('/');
            }

            if (splitPosition == -1)
            {
                this.name = uri;
                this.@namespace = String.Empty;
            }
            else
            {
                this.name = uri.Substring(splitPosition + 1);
                this.@namespace = uri.Remove(splitPosition);
            }

            this.uri = uri;
        }
    }
}
