﻿//  ------------------------------------------------------------------------
//  <copyright file="Sparql.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>28.02.2016 20:54</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using PetrSU.SmartSlog.NativeStructures;

namespace PetrSU.SmartSlog
{
    public class SparqlResultRow
    {
        /// <summary>
        /// Value type of the binding variable. 
        /// </summary>
        public List<int> types;

        /// <summary>
        /// Value of the binding variable.
        /// </summary>
        public List<string> values;

        public SparqlResultRow(IntPtr row)
        {
            if (row.ToInt32().Equals(0))
                return;   

            sslog_sparql_result_row e = (sslog_sparql_result_row)Marshal.PtrToStructure(row, typeof(sslog_sparql_result_row));

            types = MarshalHelp.PtrToIntArray(e.types);

            values = MarshalHelp.PtrToStringArray(e.values);
        }

    }


    public class SparqlResult
    {
        /// <summary>
        /// Number of binding variables in the result (columns).
        /// </summary>
        public int bindings_count;

        /// <summary>
        /// Number of rows in the result
        /// </summary>   
        public int rows_count;

        /// <summary>
        /// Names of the binding variable (columns names).
        /// </summary>
        public List<string> names;

        public List<SparqlResultRow> rows;

        public SparqlResult(IntPtr sslog_result)
        {
            if (sslog_result.ToInt32().Equals(0))
                return;    

            sslog_sparql_result e = (sslog_sparql_result)Marshal.PtrToStructure(sslog_result, typeof(sslog_sparql_result));

            bindings_count = e.bindings_count;
            rows_count = e.rows_count;

            names = MarshalHelp.PtrToStringArray(e.names);

            rows = new List<SparqlResultRow>();
            for (int i = 0; i < rows_count; ++i)
            {
                IntPtr s = Marshal.ReadIntPtr(e.rows, i * IntPtr.Size);
                rows.Add(new SparqlResultRow(s));
            }
        }
    }

}
