﻿//  ------------------------------------------------------------------------
//  <copyright file="SubscriptionEventArgs.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>02.02.2016 9:17</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace PetrSU.SmartSlog
{
    /// <summary>
    /// Class is used to pass data about subscription to the <see cref="PetrSU.SmartSlog.Subscription"/> class event.
    /// This argumets include updated data after receiveng subscription and updating subscription. 
    /// In this class there is no public constructors you can't create it manually.
    /// </summary>
    /// <seealso cref="PetrSU.SmartSlog.Subscription"/>
    public class SubscriptionEventArgs : EventArgs
    {
        /// <summary>
        /// Dictionary to store individuals and updated properties.
        /// </summary>
        private Dictionary<Individual, List<Property>> updates = new Dictionary<Individual, List<Property>>();

        /// <summary>
        /// List of updated individuals.
        /// </summary>
        private List<Individual> individuals = new List<Individual>();

        /// <summary>
        /// Initializes a new instance of the SubscriptionEventArgs class. 
        /// It is used only to prevent public creation.
        /// </summary>
        internal SubscriptionEventArgs()
        {
        }

        /// <summary>
        /// Gets last changes that was made after last indictaion or after session subscription (first synchronization).
        /// </summary>
        public bool Unsubscribe
        {
            get;
            set;
        }
        

        /// <summary>
        /// Gets individuals.
        /// </summary>
        /// <returns>Lisst with individuals.</returns>
        /// <seealso cref="PetrSU.SmartSlog.Individual"/>
        public ReadOnlyCollection<Individual> GetIndividuals()
        {
            return new ReadOnlyCollection<Individual>(individuals);
        }

        /// <summary>
        /// Gets properties if given individual.
        /// </summary>
        /// <param name="individual">Individual that properties you need to get.</param>
        /// <returns>List with properties.</returns>
        public ReadOnlyCollection<Property> GetProperties(Individual individual)
        {
            List<Property> properties = null;

            if (updates.TryGetValue(individual, out properties) == false)
            {
                return new ReadOnlyCollection<Property>(new List<Property>());
            }

            return new ReadOnlyCollection<Property>(properties);
        }

        /// <summary>
        /// Add new data.
        /// </summary>
        /// <param name="individual">Individual for adding.</param>
        /// <param name="properties">Individual's properties.</param>
        /// <seealso cref="PetrSU.SmartSlog.Individual"/>
        /// <seealso cref="PetrSU.SmartSlog.Property"/>
        internal void Add(Individual individual, List<Property> properties)
        {
            updates.Add(individual, properties);
            individuals.Add(individual);
        }
    }
}
