//  ------------------------------------------------------------------------
//  <copyright file="Wife.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>22.06.2016 11:24</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading;

namespace PetrSU.SmartSlog.Demo.Drinkers
{
    /// <summary>
    /// This class is a part of a Drinkers example. In this example there are 2 KP, one(Timo) drinks beer and publish 
    /// liters, also it subscribes to sms from his Wife (second KP). Wife KP gets notification about liters 
    /// and checks that Timo did not drink much beer.
    /// </summary>
    public class Wife
    {
        /// <summary>
        /// Number of liters that can be drank by Timo.
        /// </summary>
        private const int MaxLiters = 3;

        /// <summary>
        /// Name of node. This name is used while joining to the smart space.
        /// </summary>
        private static string nodeName = "WifeConsumer";

        /// <summary>
        /// Name of the smart space. Now always 'X'.
        /// </summary>
        private static string smartSpaceName = "X";

        /// <summary>
        /// Address of the smart space, SIB.
        /// </summary>
        private static string address = "194.85.173.9"; // PetrSU 
        ////private static string address = "137.204.143.19";   // mml.arces.unibo.it 
        
        /// <summary>
        /// Port of the smart space, SIB.
        /// </summary>
        private static int port = 10011;

        /// <summary>
        /// Name of Timo.
        /// </summary>
        private static string name = "Timo";

        /// <summary>
        /// Surname of Timo.
        /// </summary>
        private static string surname = "Ericsson";

        /// <summary>
        /// Telephone number of Timo. it is used as 'to' field for SMS. 
        /// </summary>
        private static string telNumber = "87236482";

        /// <summary>
        /// Telephone number of Wife, it is Used as 'from' field for SMS.
        /// </summary>
        private static string wifeTelNumber = "732468";

        /// <summary>
        /// UUID of the sms individual. Individual with this UUID is used
        /// as sms. 
        /// </summary>
        private static string smsUUID = "SMSDrinkers";

        /// <summary>
        /// Main method starts KP. It finds Timo in the smart space, subscribes to Timo's properties
        /// and in the end sends SMS.
        /// </summary>
        /// <param name="args">Programm arguments(not used)</param>
        public static void Main(string[] args)
        {
            Node node = new Node(nodeName, smartSpaceName, address, port);
            
            OntologyStructure.RegisterOntology(node);

            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Console.WriteLine("Wife joined to the smart space.");
            Console.WriteLine("Press enter to continue...");
            Console.ReadLine();

            // Create pattern to search Drinker adding to it a name and a surname. 
            //Pattern patter = node.CreatePattern(OntologyStructure.Man);

            //patter.AddProperty(OntologyStructure.Fname, name);
            //patter.AddProperty(OntologyStructure.Lname, surname);

            // Get all individuals from the smart space by pattern.
            // If there are many or 0 Drinkers in the smart space then we leve it and 
            // program ends.
            Collection<Individual> timos = node.GetIndividuals(OntologyStructure.Man);

            if (timos.Count == 0)
            {
                Console.WriteLine("There are no Timos in the smart space.");
                LeaveSmartSpace(node);
                Console.ReadLine();
                return;
            }
            else if (timos.Count > 1)
            {
                Console.WriteLine("There are many Timos in the smart space.");
                LeaveSmartSpace(node);
                Console.ReadLine();
                return;
            }

            Individual timo = timos[0];
            
            node.Populate(timo);

            if (name.Equals(timo.GetDataProperty(OntologyStructure.Fname)) && surname.Equals(timo.GetDataProperty(OntologyStructure.Lname)))
                Console.WriteLine("Timo found.");
            else
            {
                Console.WriteLine("There are no Timos in the smart space.");
                LeaveSmartSpace(node);
                Console.ReadLine();
                return;
            }

            // Create a subscription, this subscription has Drinker with properties about a drink and liters. 
            Subscription subscSession = CreateSession(node, timo);

            try
            {
                node.Subscribe(subscSession);
            }
            catch(Exception ex)
            {
                Console.WriteLine("Can't subscribe: " + ex.ToString());
                LeaveSmartSpace(node);
                return;
            }

            // Wait while Drinker starts to drink beer.
            while (true)
            {
                if (timo.GetDataProperty(OntologyStructure.Drinks) == "Beer")
                {
                    break;
                }

                Console.WriteLine("Waiting Timo...");
                Thread.Sleep(2000);

                node.WaitSubscribe(subscSession);
            }
            
            Console.WriteLine("Haaa, Timo starts to drink");

            bool isSmsSent = false;

            // Check Drinker's liters and send SMS. 
            while (true)
            {
                int liters = GetLiters(timo);
                if (isSmsSent == false && liters > MaxLiters)
                {
                    Console.WriteLine("It's time to kill somebody... ");
                    SendSMS(node);

                    Console.WriteLine("SMS is sent");
                    isSmsSent = true;
                }
                else if (timo.GetDataProperty(OntologyStructure.Drinks) == null)
                {
                    Console.WriteLine("Timo, come to me!!!");
                    Thread.Sleep(3000);
                    break;
                }

                Thread.Sleep(2000);
                node.WaitSubscribe(subscSession);
            }

            //node.Unsubscribe(subscSession);

            LeaveSmartSpace(node);
            Console.ReadLine();
        }

        /// <summary>
        /// Creates subscription session and adds to it Timo individual with properties about drinks and liters.
        /// </summary>
        /// <param name="node">Node to create subscription.</param>
        /// <param name="timo">Individual for subscription.</param>
        /// <returns>New created subscription session.</returns>
        private static Subscription CreateSession(Node node, Individual timo)
        {
            Subscription container = node.CreateSubscription(isAsynchronous: false);

            List<Property> properties = new List<Property>()
            {
                OntologyStructure.Drinks, 
                OntologyStructure.Numberofdrinks
            };

            container.Add(timo, properties);

            return container;
        }

        /// <summary>
        /// Leaves the smart space and handle exception on error.
        /// </summary>
        /// <param name="node">Node for leaving.</param>
        private static void LeaveSmartSpace(Node node)
        {
            try
            {
                node.Leave();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't leave the smart space.\n" + e.StackTrace);
            }
        }

        /// <summary>
        /// Gets liters from an individual-drinker and parse it to integer.
        /// </summary>
        /// <param name="timo">Drinker - individual.</param>
        /// <returns>Number of liters.</returns>
        private static int GetLiters(Individual timo)
        {
            int liters = 0;
            try
            {
                liters = int.Parse(timo.GetDataProperty(OntologyStructure.Numberofdrinks));
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't parse Timo's liters. He is really good! (" + e.Message + ")");
            }

            return liters;
        }

        /// <summary>
        /// Create new individual-SMS set properties (telephone numbers) and sends it, insert to the smart space.
        /// </summary>
        /// <param name="node">Node to insert.</param>
        private static void SendSMS(Node node)
        {
            Individual sms = node.CreateIndividual(OntologyStructure.SMS, smsUUID);
            node.Remove(sms);

            sms.InsertProperty(OntologyStructure.From, wifeTelNumber);
            sms.InsertProperty(OntologyStructure.To, telNumber);

            node.Insert(sms);
        }
    }
}
