//  ------------------------------------------------------------------------
//  <copyright file="FoafGpsOntology.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>22.06.2016 11:24</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------
#define INCLUDE_PROPERTY_LAT
#define INCLUDE_PROPERTY_LOCATION
#define INCLUDE_PROPERTY_SURNAME
#define INCLUDE_PROPERTY_LONG
#define INCLUDE_PROPERTY_ALT
#define INCLUDE_PROPERTY_FIRSTNAME
#define INCLUDE_CLASS_POINT
#define INCLUDE_CLASS_PERSON

using System.Collections.Generic;
using PetrSU.SmartSlog;

namespace PertSU.SmartSlog.Demo.Gps
{
    public static class OntologyStructure
    {
        static OntologyStructure()
        {
            List<Property> properties = new List<Property>();
            List<OntologyClass> superclasses = new List<OntologyClass>();

#if INCLUDE_PROPERTY_LAT
            Lat = new Property("http://www.w3.org/2003/01/geo/wgs84_pos#lat",
                maxCardinality: 1, minCardinality: -1,
                isObject: false);
#endif
#if INCLUDE_PROPERTY_LOCATION
            Location = new Property("http://www.w3.org/2003/01/geo/wgs84_pos#location",
                maxCardinality: -1, minCardinality: -1,
                isObject: true);
#endif
#if INCLUDE_PROPERTY_SURNAME
            Surname = new Property("http://xmlns.com/foaf/0.1/surname",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);
#endif
#if INCLUDE_PROPERTY_LONG
            Long = new Property("http://www.w3.org/2003/01/geo/wgs84_pos#long",
                maxCardinality: 1, minCardinality: -1,
                isObject: false);
#endif
#if INCLUDE_PROPERTY_ALT
            Alt = new Property("http://www.w3.org/2003/01/geo/wgs84_pos#alt",
                maxCardinality: 1, minCardinality: -1,
                isObject: false);
#endif
#if INCLUDE_PROPERTY_FIRSTNAME
            FirstName = new Property("http://xmlns.com/foaf/0.1/firstName",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);
#endif




#if INCLUDE_CLASS_POINT
#if INCLUDE_PROPERTY_LAT
            properties.Add(Lat);
#endif
#if INCLUDE_PROPERTY_LOCATION
            properties.Add(Location);
#endif
#if INCLUDE_PROPERTY_ALT
            properties.Add(Alt);
#endif
#if INCLUDE_PROPERTY_LONG
            properties.Add(Long);
#endif


        Point = new OntologyClass("http://www.w3.org/2003/01/geo/wgs84_pos#Point", properties, superclasses);
        properties = new List<Property>();
        superclasses = new List<OntologyClass>();
#endif
#if INCLUDE_CLASS_PERSON
#if INCLUDE_PROPERTY_LAT
            properties.Add(Lat);
#endif
#if INCLUDE_PROPERTY_SURNAME
            properties.Add(Surname);
#endif
#if INCLUDE_PROPERTY_LOCATION
            properties.Add(Location);
#endif
#if INCLUDE_PROPERTY_FIRSTNAME
            properties.Add(FirstName);
#endif
#if INCLUDE_PROPERTY_ALT
            properties.Add(Alt);
#endif
#if INCLUDE_PROPERTY_LONG
            properties.Add(Long);
#endif


        Person = new OntologyClass("http://xmlns.com/foaf/0.1/Person", properties, superclasses);
        properties = new List<Property>();
        superclasses = new List<OntologyClass>();
#endif



        }

#if INCLUDE_CLASS_POINT
        public static OntologyClass Point {get; private set;}
#endif
#if INCLUDE_CLASS_PERSON
        public static OntologyClass Person {get; private set;}
#endif


#if INCLUDE_PROPERTY_LAT
       public static Property Lat {get; private set;}
#endif
#if INCLUDE_PROPERTY_LOCATION
       public static Property Location {get; private set;}
#endif
#if INCLUDE_PROPERTY_SURNAME
       public static Property Surname {get; private set;}
#endif
#if INCLUDE_PROPERTY_LONG
       public static Property Long {get; private set;}
#endif
#if INCLUDE_PROPERTY_ALT
       public static Property Alt {get; private set;}
#endif
#if INCLUDE_PROPERTY_FIRSTNAME
       public static Property FirstName {get; private set;}
#endif


        public static void RegisterOntology(ISession register)
        {
            if (register == null)
            {
                throw new System.ArgumentNullException("register");
            }

#if INCLUDE_PROPERTY_LAT
            register.RegisterEntity(Lat);
#endif
#if INCLUDE_PROPERTY_LOCATION
            register.RegisterEntity(Location);
#endif
#if INCLUDE_PROPERTY_SURNAME
            register.RegisterEntity(Surname);
#endif
#if INCLUDE_PROPERTY_LONG
            register.RegisterEntity(Long);
#endif
#if INCLUDE_PROPERTY_ALT
            register.RegisterEntity(Alt);
#endif
#if INCLUDE_PROPERTY_FIRSTNAME
            register.RegisterEntity(FirstName);
#endif




#if INCLUDE_CLASS_POINT
        register.RegisterEntity(Point);
#endif
#if INCLUDE_CLASS_PERSON
        register.RegisterEntity(Person);
#endif



        }
    }
}

