﻿//  ------------------------------------------------------------------------
//  <copyright file="Consumer.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>17.05.2016 15:31</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using PetrSU.SmartSlog;
using PetrSU.SmartSlog.Demo.HelloWorld;

namespace PertSU.SmartSlog.Demo.HelloWorld
{
    class Consumer : DemoKp
    {
        public Consumer() { }

        public Consumer(string[] args)
        {
            Init(args);
            node = new Node(nodeName, SmartSpaceName, address, port);
        }

        /// <summary>
        /// Just starts the programm: create new Person and starts activity.
        /// Parameters:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        /// </summary>
        /// <param name="args">Parameters for the programm.</param>
        static void Main(string[] args)
        {
            Consumer publisher = new Consumer(args);

            publisher.Start();

            Console.ReadLine();
        }

        private void Start()
        {
            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            OntologyStructure.RegisterOntology(node.Session);
            
            GetData(node);

            node.Leave();
        }


        /// <summary>
        /// Gets all individuals from the smart space and writes property value.
        /// </summary>
        /// <param name="node">Node to communicate with the smat space.</param>
        private static void GetData(Node node)
        {
            Collection<Individual> individuals = node.GetIndividuals(OntologyStructure.Universum);
            foreach (Individual individual in individuals)
            {
                string value = (string)node.GetProperty(individual, OntologyStructure.Has);
                Console.WriteLine("Individual universum has property: '{0}'", value);
            }
        }
    }
}
