﻿//  ------------------------------------------------------------------------
//  <copyright file="DemoKp.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>17.05.2016 15:31</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System.Linq;
using PetrSU.SmartSlog;

namespace PertSU.SmartSlog.Demo.HelloWorld
{
    public class DemoKp
    {
        /// <summary>
        /// Name of the smart space, currently it is always 'X'.
        /// </summary>
        public const string SmartSpaceName = "X";

        /// <summary>
        /// Name of the KP in the smart space.
        /// </summary>
        public string nodeName = "HelloWorldPublisher";

        /// <summary>
        /// IP addres of the SIB in the smart space.
        /// </summary>
        public string address = "194.85.173.9"; // PetrSU 

        /// <summary>
        /// Port of the SIB.
        /// </summary>
        public int port = 10011;

        /// <summary>
        /// Base URI for triple.
        /// </summary>
        public static string namespaceUri = "http://X/Hello/#";

        /// <summary>
        /// Subject of the triple.
        /// </summary>
        public static string subject = namespaceUri + "KP";

        /// <summary>
        /// Predicate of the triple.
        /// </summary>
        public static string predicate = namespaceUri + "says";

        /// <summary>
        /// Word-flag to stop working and publishing data to the smart space.
        /// </summary>
        public static string exitWord = "exit";

        /// <summary>
        /// Node for accessing the smart space.
        /// </summary>
        public Node node;


        /// <summary>
        /// Creates and initializes person object. 
        /// There are some parameters that can be used:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        ///  It creates hadler for CTR+C (@see CancelHandler).
        /// </summary>
        /// <param name="args">Array with parameters.</param>
        /// <returns>New person object.</returns>
        public void Init(string[] args)
        {
            for (int i = args.Count() - 1; i > 0; i -= 2)
            {
                string value = args[i];
                string key = args[i - 1];

                switch (key)
                {
                    case "--name":
                    case "-n":
                        this.nodeName = value;
                        break;
                    case "--address":
                    case "-a":
                        this.address = value;
                        break;
                    case "--port":
                    case "-p":
                        int port = 0;

                        if (int.TryParse(value, out port) == true)
                        {
                            this.port = port;
                        }

                        break;
                }
            }
        }
    }
}
