﻿//  ------------------------------------------------------------------------
//  <copyright file="Publisher.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>17.05.2016 15:31</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using System.Linq;
using PetrSU.SmartSlog;
using PetrSU.SmartSlog.Demo.HelloWorld;

namespace PertSU.SmartSlog.Demo.HelloWorld
{
    class Publisher : DemoKp
    {
        public Publisher() {}

        public Publisher(string[] args)
        {
            Init(args);
            node = new Node(nodeName, SmartSpaceName, address, port);
        }

        /// <summary>
        /// Just starts the programm: create new Person and starts activity.
        /// Parameters:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        /// </summary>
        /// <param name="args">Parameters for the programm.</param>
        static void Main(string[] args)
        {
            Publisher publisher = new Publisher(args);

            publisher.Start();

            Console.ReadLine();
        }

        private void Start()
        {
            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            OntologyStructure.RegisterOntology(node.Session);

            ProcessPubishing(node);

            node.Leave();
        }


        /// <summary>
        /// Publishes information to the smart space.
        /// </summary>
        /// <remarks>
        /// Creates triples for removing and adding. 
        /// The object of the inserting triple is reading from the sdin.
        /// If the user input 'exit' keyword then it publishes it, waits for a some time, 
        /// cleans the smart space and ends working.
        /// </remarks>
        /// <param name="node">Note to access smart space.</param>
        static void ProcessPubishing(Node node)
        {
            Collection<Individual> individuals = node.GetIndividuals(OntologyStructure.Universum);

            Individual universum = individuals.FirstOrDefault();

            string oldValue = null;

            if (universum == null)
            {
                universum = node.CreateIndividual(OntologyStructure.Universum);
                node.Insert(universum);
            }
            else
            {
                oldValue = (string)node.GetProperty(universum, OntologyStructure.Has);
            }

            while (true)
            {
                Console.Write("\nSay something: ");
                Console.Out.Flush();

                string value = Console.ReadLine();

                if (oldValue == null)
                {
                    node.InsertProperty(universum, OntologyStructure.Has, value);
                }
                else
                {
                    node.UpdateProperty(universum, OntologyStructure.Has, oldValue, value);
                }

                oldValue = value;

                if (value == exitWord)
                {
                    Console.WriteLine("Stopping...");

                    // Removes triples - clean SS.
                    // Waits other KP (without subscription) to get 'exit' triple. 
                    System.Threading.Thread.Sleep(5000);
                    node.Remove(universum);
                    break;
                }
            }
        }
    }
}
