﻿//  ------------------------------------------------------------------------
//  <copyright file="ConsumerKP.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>22.06.2016 10:36</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace PetrSU.SmartSlog.Demo.HelloWorldSubscribe
{
    /// <summary>
    /// This class is a part of example 'HelloWorld' with subscription. 
    /// It is cunsumer, who receive data of some property by subscription.
    /// </summary>
    public class ConsumerKP
    {
        /// <summary>
        /// Name of node. This name is used while joining to the smart space.
        /// </summary>
        private static string nodeName = "HelloConsumerSubs";

        /// <summary>
        /// Name of the smart space. Now always 'X'.
        /// </summary>
        private static string smartSpaceName = "X";

        /// <summary>
        /// Address of the smart space, SIB.
        /// </summary>
        private static string address = "194.85.173.9"; // PetrSU 
        ////private static string address = "137.204.143.19";   // mml.arces.unibo.it 

        /// <summary>
        /// Port of the smart space, SIB.
        /// </summary>
        private static int port = 10011;

        /// <summary>
        /// Starts KP and subscription.
        /// </summary>
        /// <param name="args">Programm arguments(not used).</param>
        public static void Main(string[] args)
        {
            Node node = new Node(nodeName, smartSpaceName, address, port);

            OntologyStructure.RegisterOntology(node.Session);

            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Console.WriteLine("ConsumerKP has joined to the smart space.");
            Console.WriteLine("Press Enter after inserting some individual...");
            Console.ReadLine();

            StartSubscription(node);

            try
            {
                node.Leave();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't leave the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Console.WriteLine("Smart space is leaved.");

            Console.ReadLine();
        }

        /// <summary>
        /// Subscibes to property 'has' and output changes of the property.
        /// Unsubscribe from property when 'exit' has been received.
        /// </summary>
        /// <param name="node">Node for subscription.</param>
        private static void StartSubscription(Node node)
        {
            Collection<Individual> individuals = node.GetIndividuals(OntologyStructure.Universum);

            Individual universum = individuals.FirstOrDefault();

            // If there are no individuals, then do nothing.
            if (universum == null)
            {
                Console.WriteLine("No individuals...");
                return;
            }

            // Create a synchrnous subscription and add property 'has' into it.
            Subscription subscSession = node.CreateSubscription(isAsynchronous: false);
            subscSession.Add(universum, new List<Property> { OntologyStructure.Has });

            try
            {
                node.Subscribe(subscSession);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Can't subscrube: " + ex.ToString());
                return;
            }

            Console.WriteLine("Data is subscribed.");

            // Wait new data and output property value. If property value equals 'exit', 
            // then stop cheking the subscription and unsubscribe it.
            while (universum.GetDataProperty(OntologyStructure.Has) != "exit")
            {
                node.WaitSubscribe(subscSession);
                string value = universum.GetDataProperty(OntologyStructure.Has);
                Console.WriteLine("Individual universum has property: {0}", value);
            }

            node.Unsubscribe(subscSession);
        }
    }
}
