﻿//  ------------------------------------------------------------------------
//  <copyright file="PublisherKP.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>22.06.2016 10:36</created>
//  <lastedit>05.07.2016 14:38</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;

namespace PetrSU.SmartSlog.Demo.HelloWorldSubscribe
{
    /// <summary>
    /// This class is part of example 'HelloWorld' with subscription. 
    /// It is publisher, who update some property in the smart space.
    /// </summary>
    public class PublisherKP
    {
        /// <summary>
        /// Name of node. This name is used while joining to the smart space.
        /// </summary>
        private static string nodeName = "HelloPublisherSubs";

        /// <summary>
        /// Name of the smart space. Now always 'X'.
        /// </summary>
        private static string smartSpaceName = "X";

        /// <summary>
        /// Address of the smart space, SIB.
        /// </summary>
        private static string address = "194.85.173.9"; // PetrSU 
        ////private static string address = "137.204.143.19";   // mml.arces.unibo.it 

        /// <summary>
        /// Port of the smart space, SIB.
        /// </summary>
        private static int port = 10011;

        /// <summary>
        /// Connects to the SIB and gets from the smart space or creates a new 
        /// individual. Update 'has' property in the smart space. 
        /// </summary>
        /// <param name="args">Programm arguments(not used)</param>
        public static void Main(string[] args)
        {
            Node node = new Node(nodeName, smartSpaceName, address, port);

            OntologyStructure.RegisterOntology(node.Session);

            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Console.WriteLine("PublisherKP has joined to the smart space.");

            // Try to find an individual in the smart space.
            Collection<Individual> individuals = node.GetIndividuals(OntologyStructure.Universum);

            Individual universum = individuals.FirstOrDefault();

            string oldValue = null;

            // If there is no individual, then create a new and insert it to the smart space.
            if (universum == null)
            {
                universum = node.CreateIndividual(OntologyStructure.Universum);
                node.Insert(universum);
            }
            else
            {
                oldValue = (string)node.GetProperty(universum, OntologyStructure.Has);
            }

            // If individual from the smart space has not property 'has' yet, 
            // then set it with empty value.
            if (oldValue == null)
            {
                node.InsertProperty(universum, OntologyStructure.Has, "default");
                oldValue = "default";
            }

            // Read new value for property and update it in the smart space.
            // If value = 'exit', then wait (consumer received 'exit' and unsubscribed) 
            // and remove the individual from the smart space.
            while (true)
            {
                Console.Write("\nSay something or 'exit': ");
                string value = Console.ReadLine();

                node.UpdateProperty(universum, OntologyStructure.Has, oldValue, value);
                oldValue = value;

                if (value.Equals("exit") == true)
                {
                    Thread.Sleep(5000);
                    node.Remove(universum);

                    break;
                }
            }

            try
            {
                node.Leave();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't leave the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Console.WriteLine("Smart space is leaved.");

            Console.ReadLine();
        }
    }
}
