//  ------------------------------------------------------------------------
//  <copyright file="KPTemplate.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>22.06.2016 11:24</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using PetrSU.SmartSlog;

namespace Player
{
    class KP
    {
        OntologyClass VideoDevice;
        OntologyClass MobilePhone;
        OntologyClass AudioPlayer;
        OntologyClass Nothing;
        OntologyClass Thing;
        OntologyClass Device;
        OntologyClass MediaDevice;
        OntologyClass TV;

        Property DeviceName;
        Property MaxVolume;
        Property ChannelName;
        Property songName;
        Property songNum;
        Property hasMissedCalls;
        Property isCalling;
        Property isVolumeOn;
        Property CurrentVolume;
        Property Manufacturer;
        Property Resolution;
        Property Channel;

        static void Main(string[] args)
        {
            Node node = new Node("No2123", "X", "194.85.173.9", 10010);
            KP kp = new KP();
            kp.registerOntology(node);

            try
            {
                node.Join();
            }
            catch
            {
                Console.WriteLine("Can't connect to SS, press any key to continue");
                Console.ReadLine();
            }
            Console.WriteLine("Joined the SS");

            Individual mobilePhone;

            Collection<Individual> mobilePhones = node.GetIndividuals(kp.MobilePhone);

            if (mobilePhones != null)
                mobilePhone = mobilePhones.FirstOrDefault();
            else
            {   
                mobilePhone = new Individual(kp.MobilePhone, "phone1");
                node.Insert(mobilePhone);
             }
             
            string callingStatus = node.GetProperty(mobilePhone, kp.isCalling) as string;

            if (callingStatus == null)
                node.InsertProperty(mobilePhone, kp.isCalling, "no");
            else
                node.UpdateProperty(mobilePhone, kp.isCalling, callingStatus, "no");

            string income;
            bool flag = true; //no calling right now

            printMenu();
            
            while ((income = Console.ReadLine()) != "3")
            {
                if (income == "1" && flag)
                {
                    try
                    {
                        node.UpdateProperty(mobilePhone, kp.isCalling, "no", "yes");
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine("Values coudn't updated: " + ex.StackTrace);
                    }
                    Console.WriteLine("Phone is calling");
                    flag = false;
                }
                if (income == "2" && !flag)
                {
                    try
                    {
                        node.UpdateProperty(mobilePhone, kp.isCalling, "yes", "no");
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine("Values coudn't updated: " + ex.StackTrace);
                    }
                    Console.WriteLine("Phone isn't calling");
                    flag = true;
                }
                printMenu();
            }

            node.Remove(mobilePhone);
            
            node.Leave();

            Console.WriteLine("Leaved SS");

            Console.ReadLine();
        }

        static void printMenu()
        {
            Console.WriteLine("Press 1 to call");
            Console.WriteLine("Press 2 to stop calling");
            Console.WriteLine("Press 3 to exit");
        }

        private void registerOntology(Node node)
        {
            List<Property> properties = new List<Property>();
            List<OntologyClass> superclasses = new List<OntologyClass>();

            DeviceName = new Property("DeviceName",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            MaxVolume = new Property("MaxVolume",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            ChannelName = new Property("ChannelName",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            songName = new Property("songName",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            songNum = new Property("songNum",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            hasMissedCalls = new Property("hasMissedCalls",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            isCalling = new Property("isCalling",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            node.Session.RegisterEntity(isCalling);

            isVolumeOn = new Property("isVolumeOn",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            CurrentVolume = new Property("CurrentVolume",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            Manufacturer = new Property("Manufacturer",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            Resolution = new Property("Resolution",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);

            Channel = new Property("Channel",
                maxCardinality: -1, minCardinality: -1,
                isObject: false);


            properties.Add(Resolution);

            properties.Add(Manufacturer);

            properties.Add(DeviceName);

            superclasses.Add(Device);
            VideoDevice = new OntologyClass("#VideoDevice", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(VideoDevice);

            properties.Add(isVolumeOn);

            properties.Add(MaxVolume);

            properties.Add(songName);

            properties.Add(CurrentVolume);

            properties.Add(hasMissedCalls);

            properties.Add(songNum);

            properties.Add(Manufacturer);

            properties.Add(DeviceName);

            properties.Add(isCalling);

            superclasses.Add(AudioPlayer);

            superclasses.Add(MediaDevice);


            MobilePhone = new OntologyClass("#MobilePhone", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(MobilePhone);

            properties.Add(songName);

            properties.Add(songNum);

            properties.Add(Manufacturer);

            properties.Add(DeviceName);

            superclasses.Add(Device);

            AudioPlayer = new OntologyClass("#AudioPlayer", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(AudioPlayer);

            properties.Add(Resolution);

            properties.Add(isVolumeOn);

            properties.Add(MaxVolume);

            properties.Add(songName);

            properties.Add(ChannelName);

            properties.Add(CurrentVolume);

            properties.Add(hasMissedCalls);

            properties.Add(songNum);

            properties.Add(Manufacturer);

            properties.Add(Channel);

            properties.Add(DeviceName);

            properties.Add(isCalling);

            superclasses.Add(MobilePhone);

            superclasses.Add(TV);

            Nothing = new OntologyClass("http://www.w3.org/2002/07/owl#Nothing", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(Nothing);


            Thing = new OntologyClass("http://www.w3.org/2002/07/owl#Thing", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(Thing);

            properties.Add(Manufacturer);

            properties.Add(DeviceName);

            superclasses.Add(Thing);

            Device = new OntologyClass("#Device", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(Device);

            properties.Add(isVolumeOn);

            properties.Add(MaxVolume);

            properties.Add(CurrentVolume);

            properties.Add(Manufacturer);

            properties.Add(DeviceName);

            superclasses.Add(Device);


            MediaDevice = new OntologyClass("#MediaDevice", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(MediaDevice);

            properties.Add(Resolution);

            properties.Add(ChannelName);

            properties.Add(Manufacturer);

            properties.Add(Channel);

            properties.Add(DeviceName);

            superclasses.Add(VideoDevice);


            TV = new OntologyClass("#TV", properties, superclasses);
            properties = new List<Property>();
            superclasses = new List<OntologyClass>();
            node.Session.RegisterEntity(TV);
        }
    }
}

