﻿//  ------------------------------------------------------------------------
//  <copyright file="Consumer.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>28.02.2016 20:54</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using PetrSU.SmartSlog;

namespace PertSU.SmartSlog.Demo.HelloWorld
{
    class Consumer : DemoKp
    {
        public Consumer() { }

        public Consumer(string[] args)
        {
            Init(args);
            node = new Node(nodeName, SmartSpaceName, address, port);
        }

        /// <summary>
        /// Just starts the programm: create new Person and starts activity.
        /// Parameters:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        /// </summary>
        /// <param name="args">Parameters for the programm.</param>
        static void Main(string[] args)
        {
            Consumer publisher = new Consumer(args);

            publisher.Start();

            Console.ReadLine();
        }

        private void Start()
        {
            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            Select();

            node.Leave();

            Console.ReadLine();
        }

        /// <summary>
        /// Checks triples in the smart space.
        /// </summary>
        /// <remarks>
        /// Method queries triples and write information about them.
        /// If it receives a KP_EXIT_WORD as value of the object in some triple, then 
        /// it stops checking.
        /// </remarks>
        void Select()
        {
            //string query =
            //    "PREFIX sr: <http://www.cs.karelia.ru/smartroom#>"
            //    + "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
            //    + "SELECT ?sub "
            //    + "WHERE { ?sub ?pred ?obj } LIMIT 499";


//            string query = @"PREFIX foaf:  <http://xmlns.com/foaf/0.1/>
                            //SELECT ?name ?email
                            //WHERE {
                            //    {
                            //      SELECT DISTINCT ?person ?name WHERE { 
                            //        ?person foaf:name ?name 
                            //      } ORDER BY ?name 
                            //    }
                            //    OPTIONAL { ?person foaf:mbox ?email }
                            //}";

            string query = "SELECT ?sub ?pred ?obj "
                             + "WHERE { <" + subject + "> <" + predicate + "> ?obj. "
                             + "<" + subject + "> ?pred ?obj."
                             + "?sub <" + predicate + "> ?obj }";

            SparqlResult result = node.SparqlSelect(query, "sub pred obj");

            if (result == null)
            {
                Console.WriteLine("No results");
                return;
            }

            foreach(string name in result.names)
            {
                Console.Write(name + ", ");
            }

            Console.WriteLine("");

            foreach(SparqlResultRow row in result.rows)
            {
                foreach (string value in row.values)
                {
                    Console.Write(value + ", ");
                }

                Console.WriteLine("");
            }

            Console.WriteLine("");
        }
    }
}
