﻿//  ------------------------------------------------------------------------
//  <copyright file="Publisher.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>15.03.2016 13:30</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using PetrSU.SmartSlog;

namespace PertSU.SmartSlog.Demo.HelloWorldSubscription
{
    class Publisher : DemoKp
    {
        public Publisher() {}

        public Publisher(string[] args)
        {
            Init(args);
            node = new Node(nodeName, SmartSpaceName, address, port);
        }

        /// <summary>
        /// Just starts the programm: create new Person and starts activity.
        /// Parameters:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        /// </summary>
        /// <param name="args">Parameters for the programm.</param>
        static void Main(string[] args)
        {
            Publisher publisher = new Publisher(args);

            publisher.Start();

            Console.ReadLine();
        }

        private void Start()
        {
            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            ProcessPubishing(node);

            node.Leave();
        }


        /// <summary>
        /// Publishes information to the smart space.
        /// </summary>
        /// <remarks>
        /// Creates triples for removing and adding. 
        /// The object of the inserting triple is reading from the sdin.
        /// If the user input 'exit' keyword then it publishes it, waits for a some time, 
        /// cleans the smart space and ends working.
        /// </remarks>
        /// <param name="node">Note to access smart space.</param>
        static void ProcessPubishing(Node node)
        {
            // Create triple for removing data, this triple is not changing while 
            // publishing process working.    
            Triple rem_triple = Triple.CreateDetached(subject, predicate, Triple.AnyUri, RdfType.Uri, RdfType.Uri);


            // Read a string from the stdin and create a triple using this line as 
            // a value for the object. 
            // Update triple in the SS. 
            // If object equals KP_EXIT_WORD constant, then it waiting for a some time
            // and cleaning the SS.
            while (true)
            {

                Console.WriteLine("\nSay something or write '{0}':", exitWord);

                string object_text = Console.ReadLine();

                // Remove the '\n' character.
                //remove_newline(object_text);

                // Create triple for inserting.
                Triple ins_triple = Triple.CreateDetached(subject, predicate, object_text,
                        RdfType.Uri, RdfType.Literal);

                Console.WriteLine("Publishing: {0}", ins_triple.ObjectElement);

                // Removes old value and insert a new.
                node.UpdateTriple(rem_triple, ins_triple);

                if (object_text == exitWord)
                {
                    Console.WriteLine("Stopping...");

                    // Removes triples - clean SS.
                    // Waits other KP (without subscription) to get 'exit' triple. 
                    System.Threading.Thread.Sleep(5000);
                    node.RemoveTriple(rem_triple);
                    break;
                }
            }
        }
    }
}
