﻿//  ------------------------------------------------------------------------
//  <copyright file="Consumer.cs" company="PetrSU">
//  SmartSlog Library is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License as 
//  published by the Free Software Foundation; either version 2 of 
//  the License, or (at your option) any later version.
//  
//  SmartSlog Library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SmartSlog Library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor,
//  Boston, MA  02110-1301  USA
//  </copyright>
//  
//  <author>SmartSlog Team (Aleksandr A. Lomov)</author>
//  <email>mailto:lomov@cs.karelia.ru</email>
//  <author>Andrey S. Vdovenko</author>
//  <email>mailto:vdovenko@cs.karelia.ru</email>
//  <created>28.02.2016 20:54</created>
//  <lastedit>05.07.2016 14:39</lastedit>
//  ------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Threading;
using PetrSU.SmartSlog;

namespace PertSU.SmartSlog.Demo.HelloWorldSubscription
{
    /// <summary>
    /// Consumer of 'Hello World' example with subscription.
    /// </summary>
    /// <remarks>
    /// This KP make a subscription to triples using <KP - says - *> triple-pattern.
    /// It gets information about changes and outputs old and new values. 
    /// To stop the KP use CTRL+C. 
    /// </remarks>
    class Consumer : DemoKp
    {
        public Consumer() { }

        public Consumer(string[] args)
        {
            Init(args);
            node = new Node(nodeName, SmartSpaceName, address, port);
        }

        /// <summary>
        /// Just starts the programm: create new Person and starts activity.
        /// Parameters:
        ///  --name (-n): name of the KP;
        ///  --address (-a): IP address of the SIB;
        ///  --port (-p): port of the SIB. 
        /// </summary>
        /// <param name="args">Parameters for the programm.</param>
        static void Main(string[] args)
        {
            Consumer consumer = new Consumer(args);

            consumer.Start();

            Console.ReadLine();
        }

        private void Start()
        {
            try
            {
                node.Join();
            }
            catch (Exception e)
            {
                Console.WriteLine("Can't connect to the smart space.\n" + e.StackTrace);
                Console.ReadLine();
                return;
            }

            //CheckingTriples();

            Subscription subscription = MakeSubscription();

            HandleSubscription(subscription);

            node.Leave();
        }

        /// <summary>
        /// Checks triples in the smart space.
        /// </summary>
        /// <remarks>
        /// Method queries triples and write information about them.
        /// If it receives a KP_EXIT_WORD as value of the object in some triple, then 
        /// it stops checking.
        /// </remarks>
        void CheckingTriples()
        {
            ISession session = node.Session;

            // Create the triple for a query operation.
            Triple reqTriple = session.CreateDetachedTriple(subject, predicate, Triple.AnyUri,
                RdfType.Uri, RdfType.Uri);

            if (reqTriple == null)
            {
                return;
            }

            bool is_checking = true;

            while (is_checking == true)
            {
                // Remove local triples and get new from smart space.
                session.RemoveTriple(reqTriple);

                ICollection<ITriple> triples = node.QueryTriple(reqTriple);

                // Print information using triples that were receievng by the query.
                foreach (Triple triple in triples)
                {
                    Console.WriteLine("KP says: {0}", triple.ObjectElement);

                    if (triple.ObjectElement == exitWord)
                    {
                        Console.WriteLine("It's time to leave...");
                        is_checking = false;
                        break;
                    }
                }

                // Sleep for a some time between cheking.
                Thread.Sleep(2000);
            }
        }


        private Subscription MakeSubscription()
        {
            // Create the triple for a subscription operation.    
            Triple req_triple =  node.Session.CreateDetachedTriple(subject, predicate, Triple.AnyUri,
                                                                   RdfType.Uri, RdfType.Uri);

            Subscription subscription = new Subscription(node, false);
            
            subscription.Add(req_triple);

            node.Subscribe(subscription);

            return subscription;
        }


        /**
 * @brief Checks notifications from the smart space. 
 * It works while exit_flag is 0 or while error/unsubscription indication is not 
 * received.   
 *
 * @param subscription. Subscription to handle.
 */
        void HandleSubscription(Subscription subscription)
        {
            bool is_checking = true;

            Triple reqTriple = node.Session.CreateDetachedTriple(subject, predicate, Triple.AnyUri,
                                                                   RdfType.Uri, RdfType.Uri);

            while (is_checking)
            { 
                node.WaitSubscribe(subscription);

                ICollection<ITriple> triples = subscription.LastChanges.GetTriples(ActionType.Insert);

                // Print information using triples that were receievng by the query.
                foreach (Triple triple in triples)
                {
                    Console.WriteLine("KP says: {0}", triple.ObjectElement);

                    if (triple.ObjectElement == exitWord)
                    {
                        Console.WriteLine("It's time to leave...");
                        is_checking = false;
                        return;
                    }
                }

            }

            node.Unsubscribe(subscription);
        }
    }
}
